@@ -18,7 +18,6 @@ class Beam(Application):
     After initialization, the function setMesh(mesh) has to be called
     Then, solve() solves the stationary problem
     Parameters in the constructor:
-        fem: only p1 or cr1
         problemdata
     Paramaters used from problemdata:
         EI
@@ -37,16 +36,33 @@ class Beam(Application):
     def defineRhsAnalyticalSolution(self, solexact):
         def _fctu(x, y, z):
             EI = self.problemdata.params.scal_glob['EI']
-            rhs = EI * solexact.dddd(x, y, z)
+            rhs = EI * solexact.xxxx(x, y, z)
             return rhs
         return _fctu
     def defineClampedAnalyticalSolution(self, problemdata, color):
         solexact = problemdata.solexact
-        def _fctneumann(x, y, z, nx, ny, nz):
+        def _fctclamped(x, y, z, nx, ny, nz):
             rhs = solexact.d(0, x, y, z) * nx
             return rhs
-        return solexact, _fctneumann
-    def solve(self, iter, dirname): return self.static(iter, dirname)
+        return solexact, _fctclamped
+    def defineSimplySupportedAnalyticalSolution(self, problemdata, color):
+        solexact = problemdata.solexact
+        EI = self.problemdata.params.scal_glob['EI']
+        def _fctsimsupp2(x, y, z, nx, ny, nz):
+            rhs = EI*solexact.xx(x, y, z) * nx
+            return rhs
+        return solexact, _fctsimsupp2
+    def defineForcesAnalyticalSolution(self, problemdata, color):
+        solexact = problemdata.solexact
+        EI = self.problemdata.params.scal_glob['EI']
+        def _fctsimsupp2(x, y, z, nx, ny, nz):
+            rhs = EI*solexact.xx(x, y, z) * nx
+            return rhs
+        def _fctsimsupp3(x, y, z, nx, ny, nz):
+            rhs = EI*solexact.xxx(x, y, z) * nx
+            return rhs
+        return _fctsimsupp2, _fctsimsupp3
+
     def setMesh(self, mesh):
         assert mesh.dimension == 1
         super().setMesh(mesh)
@@ -54,40 +70,59 @@ class Beam(Application):
         self._checkProblemData()
         self.fem.setMesh(self.mesh)
         self.EIcell = self.compute_cell_vector_from_params('EI', self.problemdata.params)
+        self.prepareBoundary()
+    def prepareBoundary(self):
+        self.facesDir = []
+        colors = self.problemdata.bdrycond.colorsOfType("Clamped")
+        for i,color in enumerate(colors):
+            faces = self.mesh.bdrylabels[color]
+            self.facesDir.append(faces[0])
+        colors = self.problemdata.bdrycond.colorsOfType("SimplySupported")
+        for i,color in enumerate(colors):
+            faces = self.mesh.bdrylabels[color]
+            self.facesDir.append(faces[0])
+        self.faceNotNeu = []
+        colors = self.problemdata.bdrycond.colorsOfType("SimplySupported")
+        for i,color in enumerate(colors):
+            faces = self.mesh.bdrylabels[color]
+            self.faceNotNeu.append(faces[0])
+        colors = self.problemdata.bdrycond.colorsOfType("Forces")
+        for i,color in enumerate(colors):
+            faces = self.mesh.bdrylabels[color]
+            self.faceNotNeu.append(faces[0])
+
     def computeMatrix(self, coeffmass=None):
         A = self.fem.computeMatrixDiffusion(coeff=1)
         n = self.fem.nunknowns()
-        mats = np.ones(2)
-        rows = np.array([0,1])
-        # l'extrémité droite et le deuxième points !
-        cols = np.array([0,1])
-        C = sparse.coo_matrix((mats, (rows, cols)), shape=(2, n)).tocsr()
+        ndir = len(self.facesDir)
+        C1 = sparse.csr_matrix((np.ones(ndir), (np.arange(ndir), self.facesDir)), shape=(ndir, n)).tocsr()
+        nnotn = len(self.faceNotNeu)
+        C2 = sparse.csr_matrix((np.ones(nnotn), (np.arange(nnotn), self.faceNotNeu)), shape=(nnotn, n)).tocsr()
         dV = self.mesh.dV
         D = dV / self.EIcell / 4
-        # D = np.arange(1,n)
-        # print(f"{D=}")
         E = np.empty(n)
         E[:-1] = D
         E[1:] += D
-        # print(f"{E=}")
         B = sparse.diags((D, E, D), offsets=(-1,0,1), shape=(n, n))
-        # raise ValueError(f"B=\n{B.toarray()}\n")
-        return A, B, C
+        return A, B, C1, C2
     def computeRhs(self, b=None, u=None, coeffmass=None):
+        ndir = len(self.facesDir)
+        nnotn = len(self.faceNotNeu)
         if b is None:
             a = np.zeros(self.fem.nunknowns())
             b = np.zeros(self.fem.nunknowns())
-            c = np.zeros(2)
+            c = np.zeros(ndir)
+            d = np.zeros(nnotn)
         if 'rhs' in self.problemdata.params.fct_glob:
             xc, yc, zc = self.mesh.pointsc.T
             dV, simplices = self.mesh.dV, self.mesh.simplices
             fc = self.problemdata.params.fct_glob['rhs'](xc, yc, zc)
-            # print(f"{fc=}")
             self.fem.massDotCell(a, fc)
             Dmub = -dV**3/self.EIcell/24*fc
             np.add.at(b, simplices, Dmub[:, np.newaxis])
-        colors = self.problemdata.bdrycond.colorsOfType("Clamped")
         x, y, z = self.mesh.pointsf.T
+        idir=0
+        colors = self.problemdata.bdrycond.colorsOfType("Clamped")
         for i,color in enumerate(colors):
             faces = self.mesh.bdrylabels[color]
             normalsS = self.mesh.normals[faces]
@@ -96,15 +131,47 @@ class Beam(Application):
             nx, ny, nz = normalsS.T
             if not color in self.problemdata.bdrycond.fct: continue
             fct1, fct2 = self.problemdata.bdrycond.fct[color]
-            c[faces[0]] = fct1(x[faces], y[faces], z[faces])
+            c[idir] = fct1(x[faces], y[faces], z[faces])
+            idir += 1
             dn = fct2(x[faces], y[faces], z[faces], nx, ny, nz)
-            print(f"{dn=} {x[faces]=} {nx=} {faces=}")
+            cell = self.mesh.cellsOfFaces[faces[0], 0]
+            # print(f"{nx=} {faces=} {self.mesh.simplices[cell]=}")
             b[faces] += dn
-        return (a,b,c), u
+        colors = self.problemdata.bdrycond.colorsOfType("SimplySupported")
+        for i,color in enumerate(colors):
+            faces = self.mesh.bdrylabels[color]
+            normalsS = self.mesh.normals[faces]
+            dS = np.linalg.norm(normalsS,axis=1)
+            normalsS = normalsS/dS
+            nx, ny, nz = normalsS.T
+            if not color in self.problemdata.bdrycond.fct: continue
+            fct1, fct2 = self.problemdata.bdrycond.fct[color]
+            c[idir] = fct1(x[faces], y[faces], z[faces])
+            idir += 1
+            ddn = fct2(x[faces], y[faces], z[faces], nx, ny, nz)
+            cell = self.mesh.cellsOfFaces[faces[0], 0]
+            # print(f"{faces=} {self.mesh.simplices[cell]=} {self.fem.cellgrads[cell]=}")
+            a[self.mesh.simplices[cell]] -= ddn*self.fem.cellgrads[cell][:,0]
+        colors = self.problemdata.bdrycond.colorsOfType("Forces")
+        for i,color in enumerate(colors):
+            faces = self.mesh.bdrylabels[color]
+            normalsS = self.mesh.normals[faces]
+            dS = np.linalg.norm(normalsS,axis=1)
+            normalsS = normalsS/dS
+            nx, ny, nz = normalsS.T
+            if not color in self.problemdata.bdrycond.fct: continue
+            fct1, fct2 = self.problemdata.bdrycond.fct[color]
+            ddn = fct1(x[faces], y[faces], z[faces], nx, ny, nz)
+            dddn = fct2(x[faces], y[faces], z[faces], nx, ny, nz)
+            cell = self.mesh.cellsOfFaces[faces[0], 0]
+            # print(f"{faces=} {self.mesh.simplices[cell]=} {self.fem.cellgrads[cell]=}")
+            a[self.mesh.simplices[cell]] -= ddn*self.fem.cellgrads[cell][:,0]
+            a[faces] += dddn
+        return a,b,c,d
     def postProcess(self, uin):
         data = {'point':{}, 'cell':{}, 'global':{}}
         u,w,l = uin
-        print(f"{l=} {u[0]=} {u[1]=}")
+        # print(f"{l=} {u[0]=} {u[1]=}")
         data['point']['U'] = self.fem.tonode(u)
         data['point']['W'] = self.fem.tonode(w)
         if self.problemdata.solexact:
@@ -114,25 +181,32 @@ class Beam(Application):
         return data
     def _to_single_matrix(self, Ain):
         n = self.fem.nunknowns()
-        A, B, C = Ain
-        null1 = sparse.coo_matrix(([], ([], [])), shape=(n, n))
-        null2 = sparse.coo_matrix(([], ([], [])), shape=(2, n))
-        null3 = sparse.coo_matrix(([], ([], [])), shape=(2, 2))
-        A1 = sparse.hstack([null1, A.T, C.T])
-        A2 = sparse.hstack([A, B, null2.T])
-        A3 = sparse.hstack([C, null2, null3])
-        Aall = sparse.vstack([A1, A2, A3]).tocsr()
+        A, B, C1, C2 = Ain
+        n1, n2 = C1.shape[0], C2.shape[0]
+        # print(f"{n1=} {n2=}")
+        null1 = sparse.csr_matrix(([], ([], [])), shape=(n, n))
+        null2 = sparse.csr_matrix(([], ([], [])), shape=(n1, n))
+        null3 = sparse.csr_matrix(([], ([], [])), shape=(n1, n1))
+        null4 = sparse.csr_matrix(([], ([], [])), shape=(n2, n))
+        null5 = sparse.csr_matrix(([], ([], [])), shape=(n2, n2))
+        null6 = sparse.csr_matrix(([], ([], [])), shape=(n1, n2))
+        A1 = sparse.hstack([null1, A.T, C1.T, null4.T])
+        A2 = sparse.hstack([A, B, null2.T, C2.T])
+        A3 = sparse.hstack([C1, null2, null3, null6])
+        A4 = sparse.hstack([null4, C2, null6.T, null5])
+        Aall = sparse.vstack([A1, A2, A3, A4]).tocsr()
+        assert np.allclose(A.data, A.T.data)
         assert np.allclose(Aall.data, Aall.T.data)
-        print(f"A=\n{Aall.toarray()}")
-        return Aall
-    def linearSolver(self, Ain, bin, uin=None, linearsolver='umf', verbose=0):
+        # print(f"A=\n{Aall.toarray()}")
+        return Aall.tocsr()
+    def linearSolver(self, Ain, bin, uin=None, verbose=0):
         n = self.fem.nunknowns()
-        if solver == 'umf':
+        if self.linearsolver == 'spsolve':
             Aall = self._to_single_matrix(Ain)
-            ball = np.hstack((bin[0], bin[1], bin[2]))
+            ball = np.hstack((bin[0], bin[1], bin[2], bin[3]))
             uall =  splinalg.spsolve(Aall, ball, permc_spec='COLAMD')
             return (uall[:n], uall[n:2*n], uall[2*n:]), 1
-        elif solver == 'gmres':
+        else:
             raise NotImplemented()
 
 
