@@ -24,18 +24,12 @@ class Elasticity(Application):
     def material2Lame(self, material):
         E, nu = self.YoungPoisson[material]
         return self.toLame(E, nu)
-    # def setParameters(self, mu, lam):
-    #     ra
-    #     self.mu, self.lam = mu, lam
-    #     self.mufct = np.vectorize(lambda j: mu)
-    #     self.lamfct = np.vectorize(lambda j: lam)
-    #     if hasattr(self,'mesh'):
-    #         self.mucell = self.mufct(self.mesh.cell_labels)
-    #         self.lamcell = self.lamfct(self.mesh.cell_labels)
     def __init__(self, **kwargs):
         fem = kwargs.pop('fem', 'p1')
         ncomp = kwargs['problemdata'].ncomp
         self.dirichletmethod = kwargs.pop('dirichletmethod', 'strong')
+        if self.dirichletmethod != 'strong':
+            raise NotImplementedError(f"only strong implemented but {self.dirichletmethod=}")
         if fem == 'p1':
             self.fem = fems.p1sys.P1sys(ncomp=ncomp)
         elif fem == 'cr1':
@@ -43,8 +37,6 @@ class Elasticity(Application):
             self.innersides=True
         else:
             raise ValueError("unknown fem '{}'".format(fem))
-        # material = kwargs.pop('material', "Acier")
-        # self.setParameters(*self.material2Lame(material))
         super().__init__(**kwargs)
     def setMesh(self, mesh):
         super().setMesh(mesh)
@@ -136,7 +128,10 @@ class Elasticity(Application):
     def pyamg_solver_args(self, maxiter):
         return {'cycle': 'V', 'maxiter': maxiter, 'tol': 1e-12, 'accel': 'bicgstab'}
     def build_pyamg(self,A):
-        import pyamg
+        try:
+            import pyamg
+        except:
+            raise ImportError(f"*** pyamg not found {self.linearsolver=} ***")
         B = pyamg.solver_configuration(A, verb=False)['B']
         symmetry = 'nonsymmetric'
         smoother = 'gauss_seidel_nr'
