@@ -17,12 +17,13 @@ class NavierStokes(Stokes):
         self.convmethod = kwargs.pop('convmethod', 'lps')
         self.lpsparam = kwargs.pop('lpsparam', 0.01)
         self.newtontol = kwargs.pop('newtontol', 1e-8)
-        if not 'precond_p' in kwargs: 
-            kwargs['precond_p'] = 'schur@diag@4'
-        if not 'precond_v' in kwargs: 
-            defsolvers = ['lgmres']
+        if kwargs['linearsolver'] != 'spsolve' and not 'precond_p' in kwargs:
+            kwargs['precond_p'] = 'schur@diag@3@scipy_gmres'
+        if kwargs['linearsolver'] != 'spsolve' and not not 'precond_v' in kwargs:
+            defsolvers = ['scipy_lgmres', 'pyamg_gmres']
             defsolvers.append('pyamg@aggregation@none@gauss_seidel')
             defsolvers.append('pyamg@aggregation@none@schwarz')
+            # defsolvers.append('pyamg@aggregation@fgmres@gauss_seidel')
             defsolvers.append('pyamg@aggregation@fgmres@schwarz')
             kwargs['precond_v'] = defsolvers
         super().__init__(**kwargs)
@@ -30,26 +31,29 @@ class NavierStokes(Stokes):
         super().setMesh(mesh)
         self.Astokes = super().computeMatrix()
     def solve(self, dirname="Run"):
-        sdata = solvers.newtondata.StoppingData(maxiter=200, steptype='rb', nbase=2, rtol=self.newtontol)
+        sdata = solvers.newtondata.StoppingData(maxiter=200, steptype='bt', nbase=1, rtol=self.newtontol)
         return self.static(dirname=dirname, mode='newton',sdata=sdata)
     def computeForm(self, u):
         # if not hasattr(self,'Astokes'): self.Astokes = super().computeMatrix()
-        d = super().matrixVector(self.Astokes,u)
+        # d = super().matrixVector(self.Astokes,u)
+        d = self.Astokes.matvec(u)
         # d = super().computeForm(u)
         v = self._split(u)[0]
         dv = self._split(d)[0]
         self.computeFormConvection(dv, v)
-        # self.femv.computeFormRtPenaly(dv, v)
+        # self.femv.computeFormHdivPenaly(dv, v, self.hdivpenalty)
         self.timer.add('form')
         return d
     def computeMatrix(self, u=None):
         # if not hasattr(self,'Astokes'): self.Astokes = super().computeMatrix()
-        X = [A.copy() for A in self.Astokes]
+        if u is None: return self.Astokes
+        # X = [A.copy() for A in self.Astokes]
         # X = super().computeMatrix(u)
-        if u is None: return X
+        X = self.Astokes.copy()
         v = self._split(u)[0]
-        X[0] += self.computeMatrixConvection(v)
-        # X[0] += self.femv.computeMatrixRtPenaly()
+        # X[0] += self.computeMatrixConvection(v)
+        X.A += self.computeMatrixConvection(v)
+        # X[0] += self.femv.computeMatrixHdivPenaly(self.hdivpenalty)
         self.timer.add('matrix')
         return X
     def computeFormConvection(self, dv, v):
@@ -66,11 +70,9 @@ class NavierStokes(Stokes):
         self.femv.massDotBoundary(dv, vdir, colors=colorsdirichlet, ncomp=self.ncomp, coeff=np.minimum(self.convdata.betart, 0))
         for icomp in range(dim):
             self.femv.fem.computeFormConvection(dv[icomp::dim], v[icomp::dim], self.convdata, method=self.convmethod, lpsparam=self.lpsparam)
-
     def computeMatrixConvection(self, v):
         A = self.femv.fem.computeMatrixConvection(self.convdata, method=self.convmethod, lpsparam=self.lpsparam)
         return self.femv.matrix2systemdiagonal(A, self.ncomp).tocsr()
-
     def computeBdryNormalFluxNitsche(self, v, p, colors):
         ncomp, bdryfct = self.ncomp, self.problemdata.bdrycond.fct
         flux = super().computeBdryNormalFluxNitsche(v,p,colors)
@@ -89,7 +91,8 @@ class NavierStokes(Stokes):
         #     yv = self._split(y)[0]
         #     self.A[0] = tools.matrix.addRankOne(self.A[0], step*dv, yv, relax=1)          
         try:
-            u, niter = self.linearSolver(self.A, bin=b, uin=None, linearsolver=self.linearsolver, rtol=rtol)
+            # u, niter = self.linearSolver(self.A, bin=b, uin=None, linearsolver=self.linearsolver, rtol=rtol)
+            u, niter = self.linearSolver(self.A, bin=b, uin=None, rtol=rtol)
         except Warning:
             raise ValueError(f"matrix is singular {self.A.shape=} {self.A.diagonal()=}")
         self.timer.add('solve')
@@ -97,7 +100,7 @@ class NavierStokes(Stokes):
     def computePrec(self, b, u=None):
         self.A = self.computeMatrix(u=u) 
         try:
-            u, niter = self.linearSolver(self.A, bin=b, uin=u, linearsolver=self.linearsolver, rtol=0.1)
+            u, niter = self.linearSolver(self.A, bin=b, uin=u, rtol=0.1)
         except Warning:
             raise ValueError(f"matrix is singular {self.A.shape=} {self.A.diagonal()=}")
         self.timer.add('solve')
