@@ -1,3 +1,5 @@
+import copy
+
 from matplotlib import colors
 import numpy as np
 import scipy.sparse as sparse
@@ -5,7 +7,7 @@ import scipy.sparse.linalg as splinalg
 from simfempy import fems, tools, solvers
 from simfempy.applications.application import Application
 from simfempy.tools.analyticalfunction import analyticalSolution
-from simfempy.tools import npext
+from simfempy.solvers import linalg
 from functools import partial
 
 #=================================================================#
@@ -15,21 +17,84 @@ class Stokes(Application):
     def __format__(self, spec):
         if spec=='-':
             repr = f"{self.femv=} {self.femp=}"
-            repr += f"\tlinearsolver={self.linearsolver}"
+            if self.linearsolver=='spsolve': return repr + self.linearsolver
+            ls = '@'.join([str(v) for v in self.linearsolver.values()])
+            vs = '@'.join([str(v) for v in self.solver_v.values()])
+            # print(f"{self.solver_p=}")
+            ps = '@'.join([str(v) for v in self.solver_p.values()])
+            repr += f"\tlinearsolver={ls} V:{vs} P:{ps}"
             return repr
         return self.__repr__()
+    def _solvername_to_dict(self, name, type=None):
+        nsp = name.split('@')
+        if type == 'linearsolver':
+            if len(nsp) != 4:
+                raise ValueError(f"*** need 'linearsolver' in the form 'method@prec@maxiter@disp' got '{name}'")
+            return {'method': nsp[0], 'prec': nsp[1], 'maxiter': int(nsp[2]), 'disp': int(nsp[3])}
+        elif type == 'solver_v':
+            if nsp[0]=='pyamg':
+                if len(nsp)!=6:
+                    raise ValueError(f"*** need 'solver_v' in the form 'pyamg@pyamgtype@accel@smoother@maxiter@disp'")
+                return {'method':nsp[0], 'pyamgtype':nsp[1], 'accel':nsp[2], 'smoother':nsp[3],
+                            'maxiter':int(nsp[4]), 'disp':int(nsp[5])}
+            else:
+                return {'method': nsp[0], 'prec': nsp[1], 'maxiter': int(nsp[2]), 'disp': int(nsp[3])}
+        elif type == 'solver_p':
+            if nsp[0] == 'scale': return {'type': 'scale'}
+            if len(nsp)<2:
+                raise ValueError(f"*** need 'solver_p' in the form 'tye@method@...@disp'")
+            elif nsp[1]=='pyamg':
+                if len(nsp) != 7:
+                    raise ValueError(
+                        f"*** need 'solver_p' in the form 'type@pyamg@pyamgtype@accel@smoother@maxiter@disp'")
+                return {'type':nsp[0], 'method':nsp[1], 'pyamgtype':nsp[2], 'accel':nsp[3], 'smoother':nsp[4],
+                        'maxiter':int(nsp[5]), 'disp':int(nsp[6])}
+            else:
+                if len(nsp) != 5:
+                    raise ValueError(
+                        f"*** need 'solver_p' in the form 'type@method@prec@maxiter@disp'")
+                return {'type':nsp[0], 'method': nsp[1], 'prec': nsp[2], 'maxiter': int(nsp[3]), 'disp': int(nsp[4])}
+        else: raise ValueError(f"*** unknown {type=}")
     def __init__(self, **kwargs):
-        self.dirichlet_nitsche = 10
-        self.dirichletmethod = kwargs.pop('dirichletmethod', 'nitsche')
+        if 'dirichletmethod' in kwargs and kwargs['dirichletmethod']=='nitsche':
+            # correct value ?!
+            kwargs['nitscheparam'] = 10
+        self.dirichletmethod = kwargs.get('dirichletmethod', 'nitsche')
         self.problemdata = kwargs.pop('problemdata')
         self.ncomp = self.problemdata.ncomp
-        self.femv = fems.cr1sys.CR1sys(self.ncomp)
+        self.femv = fems.cr1sys.CR1sys(self.ncomp, kwargs)
         self.femp = fems.d0.D0()
         self.hdivpenalty = kwargs.pop('hdivpenalty', 0)
-        if not 'linearsolver' in kwargs: kwargs['linearsolver'] = 'gcrotmk_0'
-        self.precond_p = kwargs.pop('precond_p', 'scale')
-        # self.precond_p = kwargs.pop('precond_p', 'schur@diag@1')
-        self.precond_v = kwargs.pop('precond_v', 'pyamg@aggregation@none@gauss_seidel')
+        self.divdivparam = kwargs.pop('divdivparam', 0)
+        self.scalels = kwargs.pop('scalels', False)
+        if not 'linearsolver' in kwargs:
+            linearsolver_def = {'method': 'pyamg_gmres', 'prec':'full', 'maxiter': 200, 'disp':0}
+            kwargs['linearsolver'] = linearsolver_def
+        else:
+            linearsolver = kwargs['linearsolver']
+            if not isinstance(linearsolver, str):
+                raise ValueError(f"*** need 'linearsolver' as str")
+            if not kwargs['linearsolver'] == 'spsolve':
+                kwargs['linearsolver'] = self._solvername_to_dict(linearsolver, type='linearsolver')
+        if not 'solver_p' in kwargs:
+            self.solver_p = {'type': 'scale'}
+        else:
+            solver_p = kwargs.pop('solver_p')
+            if isinstance(solver_p, str):
+                self.solver_p = self._solvername_to_dict(solver_p, type='solver_p')
+                # print(f"???? {self.solver_p=}")
+            else:
+                raise ValueError(f"*** need 'solver_p' as str")
+        if not 'solver_v' in kwargs:
+            solver_v_def = {'method': 'pyamg', 'pyamgtype': 'aggregation', 'accel': 'none', 'smoother': 'gauss_seidel',
+                            'maxiter': 1, 'disp': 0}
+            self.solver_v = solver_v_def
+        else:
+            solver_v = kwargs.pop('solver_v')
+            if isinstance(solver_v, str):
+                self.solver_v = self._solvername_to_dict(solver_v, type='solver_v')
+            else:
+                raise ValueError(f"*** need 'solver_v' in the form 'method@prec@maxiter@disp'")
         super().__init__(**kwargs)
     def _zeros(self):
         nv = self.mesh.dimension*self.mesh.nfaces
@@ -49,7 +114,6 @@ class Stokes(Application):
         self.femv.setMesh(self.mesh)
         self.femp.setMesh(self.mesh)
         self.mucell = self.compute_cell_vector_from_params('mu', self.problemdata.params)
-        # self.pmean = list(self.problemdata.bdrycond.type.values()) == len(self.problemdata.bdrycond.type)*['Dirichlet']
         self.pmean = not ('Neumann' in self.problemdata.bdrycond.type.values() or 'Pressure' in self.problemdata.bdrycond.type.values())
         if self.dirichletmethod=='strong':
             assert 'Navier' not in self.problemdata.bdrycond.type.values()
@@ -186,136 +250,44 @@ class Stokes(Application):
                         data['global'][name] = self.computeBdryNormalFluxNitsche(v, p, colors)
                 else:
                     raise ValueError(f"unknown postprocess type '{type}' for key '{name}'\nknown types={types=}")
-        if hasattr(self.problemdata.postproc, "changepostproc"):
-            self.problemdata.postproc.changepostproc(data['global'])
         return data
-    def _to_single_matrix(self, Ain):
-        ncells, nfaces = self.mesh.ncells, self.mesh.nfaces
-        # print("Ain", Ain)
-        if self.pmean:
-            A, B, C = Ain
-        else:
-            A, B = Ain
-        nullP = sparse.dia_matrix((np.zeros(ncells), 0), shape=(ncells, ncells))
-        A1 = sparse.hstack([A, -B.T])
-        A2 = sparse.hstack([B, nullP])
-        Aall = sparse.vstack([A1, A2])
-        if not self.pmean:
-            return Aall.tocsr()
-        ncomp = self.ncomp
-        nullV = sparse.coo_matrix((1, ncomp*nfaces)).tocsr()
-        # rows = np.zeros(ncomp*nfaces, dtype=int)
-        # cols = np.arange(0, ncomp*nfaces)
-        # nullV = sparse.coo_matrix((np.zeros(ncomp*nfaces), (rows, cols)), shape=(1, ncomp*nfaces)).tocsr()
-        CL = sparse.hstack([nullV, C])
-        Abig = sparse.hstack([Aall,CL.T])
-        nullL = sparse.dia_matrix((np.zeros(1), 0), shape=(1, 1))
-        Cbig = sparse.hstack([CL,nullL])
-        Aall = sparse.vstack([Abig, Cbig])
-        return Aall.tocsr()
-    def matrixVector(self, Ain, x):
-        ncells, nfaces, ncomp = self.mesh.ncells, self.mesh.nfaces, self.ncomp
-        if self.pmean:
-            A, B, C = Ain
-            v, p, lam = x[:ncomp*nfaces], x[ncomp*nfaces:ncomp*nfaces+ncells], x[-1]*np.ones(1)
-            w = A.dot(v) - B.T.dot(p)
-            q = B.dot(v)+C.T.dot(lam)
-            return np.hstack([w, q, C.dot(p)])
-        else:
-            try:
-                A, B = Ain
-                v, p = x[:ncomp*nfaces], x[ncomp*nfaces:]
-                w = A.dot(v) - B.T.dot(p)
-                q = B.dot(v)
-            except:
-                raise ValueError(f" {v.shape=} {p.shape=}  {A.shape=} {B.shape=}")
-            return np.hstack([w, q])
-    def getPrecMult(self, Ain, AP, SP):
-        A, B = Ain[0], Ain[1]
-        ncells, nfaces, ncomp = self.mesh.ncells, self.mesh.nfaces, self.ncomp
-        if self.pmean: 
-            C = Ain[2].A.ravel()
-            BPCT = SP.solve(C)
-            # BPCT = SP.prec.solve(C.T.toarray(), maxiter=1, tol=1e-10)
-            # print(f"{BPCT=}")            
-            # print(f"{C.dot(BPCT)=}")
-            # CP = splinalg.inv(C.dot(BPCT))
-            CP = sparse.coo_matrix(1/C.dot(BPCT))
-            print(f"{CP.A=}")
-        if self.pmean: 
-            def pmult(x):
-                v, p, lam = x[:ncomp*nfaces], x[ncomp*nfaces:ncomp*nfaces+ncells], x[-1]*np.ones(1)
-                # return np.hstack([API.solve(v, maxiter=1, tol=1e-16), BP.dot(p), CP.dot(lam)])
-                w = AP.solve(v)
-                q = SP.solve(p-B.dot(w))
-                mu = CP.dot(lam-C.dot(q)).ravel()
-                # print(f"{mu.shape=} {lam.shape=} {BPCT.shape=}")
-                # q -= BPCT.dot(mu)
-                q -= BPCT*mu
-                # print(f"{BPCT.shape=} {mu=}")
-                # q -= mu*BPCT
-                h = B.T.dot(q)
-                w += AP.solve(h)
-                return np.hstack([w, q, mu])
-        else:
-            def pmult(x):
-                v, p = x[:ncomp*nfaces], x[ncomp*nfaces:ncomp*nfaces+ncells]
-                w = AP.solve(v)
-                # print(f"{np.linalg.norm(v)=} {np.linalg.norm(p)=}")
-                # print(f"{np.linalg.norm(w)=} {np.linalg.norm(p-B.dot(w))=}")
-                q = SP.solve(p-B.dot(w))
-                # print(f"{np.linalg.norm(q)=}")
-                h = B.T.dot(q)
-                w += AP.solve(h)
-                return np.hstack([w, q])
-        return pmult
-    def getVelocitySolver(self, A):
-        return solvers.cfd.VelcoitySolver(A, disp=0, counter="VS", solver=self.precond_v)
-        # return solvers.cfd.VelcoitySolver(A, solver='pyamg', maxiter=1)
-    def getPressureSolver(self, A, B, AP):
-        mu = self.problemdata.params.scal_glob['mu']
-        if self.pmean: assert self.precond_p == "schur"
-        if self.precond_p[:5] == "schur":
-            sp = self.precond_p.split('@')
-            if not len(sp)==3 or not(0 < int(sp[2]) < 20) or not sp[1] in solvers.cfd.prec_PressureSolverSchur:
-                raise ValueError(f"need 'schur@prec@maxiter' with prec in {solvers.cfd.prec_PressureSolverSchur}\ngot {self.precond_p}" )
-            return solvers.cfd.PressureSolverSchur(self.mesh, mu, A, B, AP, solver='lgmres', prec = sp[1], maxiter=int(sp[2]), disp=0)
-        elif self.precond_p == "diag":    
-            return solvers.cfd.PressureSolverDiagonal(A, B, prec='scale', accel='cg', maxiter=5, disp=1, counter="PS", symmetric=True)
-        elif self.precond_p == "scale":    
-            return solvers.cfd.PressureSolverScale(self.mesh, mu)
-        else:
-            raise ValueError(f"unknown {self.precond_p=}")   
-    def linearSolver(self, Ain, bin, uin=None, linearsolver='umf', verbose=0, atol=1e-14, rtol=1e-10):
-        ncells, nfaces, ncomp = self.mesh.ncells, self.mesh.nfaces, self.ncomp
-        if linearsolver == 'umf':
-            Aall = self._to_single_matrix(Ain)
+    def linearSolver(self, Ain, bin, uin=None, verbose=0, atol=1e-16, rtol=1e-10):
+        if self.linearsolver == 'spsolve':
+            Aall = Ain.to_single_matrix()
             uall =  splinalg.spsolve(Aall, bin, permc_spec='COLAMD')
             self.timer.add("linearsolve")
             return uall, 1
         else:
-            # print(f"{atol=} {rtol=}")
-            ssolver = linearsolver.split('_')
-            method=ssolver[0] if len(ssolver)>0 else 'lgmres'
-            disp=int(ssolver[1]) if len(ssolver)>1 else 0
-            nall = ncomp*nfaces + ncells
-            if self.pmean: nall += 1
-            AP = self.getVelocitySolver(Ain[0])
-            SP = self.getPressureSolver(Ain[0], Ain[1], AP)
-            matvec = partial(self.matrixVector, Ain)
-            matvecprec = self.getPrecMult(Ain, AP, SP)
-            maxiter = 20
-            S = solvers.linalg.ScipySolve(matvec=matvec, matvecprec=matvecprec, method=method, n=nall,
-                                            disp=disp, atol=atol, rtol=rtol, counter="sys", maxiter=maxiter)
+            linearsolver = copy.deepcopy(self.linearsolver)
+            solver_p = copy.deepcopy(self.solver_p)
+            # print(f"{self.solver_p=}")
+            solver_v = copy.deepcopy(self.solver_v)
+            prec = linearsolver.pop("prec", "full")
+            if solver_p['type']=='scale':
+                solver_p['coeff'] = self.mesh.dV/self.mucell
+            print(f"{self.scalels=}")
+            if self.scalels: Ain.scaleAb(bin)
+            P = linalg.SaddlePointPreconditioner(Ain, solver_v=solver_v, solver_p=solver_p, method=prec)
+            assert isinstance(self.linearsolver, dict)
+            linearsolver['counter'] = 'sys '
+            linearsolver['matvec'] = Ain.matvec
+            linearsolver['matvecprec'] = P.matvecprec
+            linearsolver['n'] = Ain.nall
+            S = linalg.getSolver(args=linearsolver)
+            maxiter = S.maxiter
             uall =  S.solve(b=bin, x0=uin)
+            if self.scalels: Ain.scaleu(uall)
             self.timer.add("linearsolve")
             it = S.counter.niter
-            if it==maxiter: 
+            if it==maxiter or np.linalg.norm(uall)<atol:
                 msg = f"*** linear system solver not converged in {maxiter=}"
-                msg += f"\n{self.precond_v=} {self.precond_p=}"
-                msg += f"\n{AP=} {SP=}"
+                msg += f"\n{np.linalg.norm(uall)=} {np.linalg.norm(bin)=}"
+                msg += f"\n{S=}"
+                msg += f"\n{P=}"
                 msg += f"\n{S.counter=}"
-                raise ValueError(msg)
+                # raise ValueError(msg)
+                print(msg)
+                it = -1
             return uall, it
     def computeRhs(self, b=None, u=None, coeffmass=None):
         b = self._zeros()
@@ -336,15 +308,16 @@ class Stokes(Application):
             vdir = self.femv.interpolateBoundary(colorsdir, self.problemdata.bdrycond.fct)
             self.computeRhsBdryNitscheDirichlet((bv,bp), colorsdir, vdir, self.mucell)
             bdryfct = self.problemdata.bdrycond.fct
-            colorspres = set(bdryfct.keys()).intersection(colorsnav)
-            if len(colorspres):
-                if not isinstance(bdryfct[next(iter(colorspres))],dict):
+            # Navier condition
+            colors = set(bdryfct.keys()).intersection(colorsnav)
+            if len(colors):
+                if not isinstance(bdryfct[next(iter(colors))],dict):
                     msg = """
                     For Navier b.c. please give a dictionary {vn:fct_scal, g:fvt_vec} with fct_scal scalar and fvt_vec a list of dim functions
                     """
-                    raise ValueError(msg+f"\ngiven: {bdryfct[next(iter(colorspres))]=}")
+                    raise ValueError(msg+f"\ngiven: {bdryfct[next(iter(colors))]=}")
                 vnfct, gfct = {}, {}
-                for col in colorspres:
+                for col in colors:
                     if 'vn' in bdryfct[col].keys() : 
                         if not callable(bdryfct[col]['vn']):
                             raise ValueError(f"'vn' must be a function. Given:{bdryfct[col]['vn']=}")
@@ -359,15 +332,16 @@ class Stokes(Application):
                 if len(gfct): 
                     gt = self.femv.interpolateBoundary(colorsnav, gfct)
                     self.computeRhsBdryNitscheNavierTangent((bv,bp), colorsnav, self.mucell, gt)
-            colorspres = set(bdryfct.keys()).intersection(colorsp)
-            if len(colorspres):
-                if not isinstance(bdryfct[next(iter(colorspres))],dict):
+            # Pressure condition
+            colors = set(bdryfct.keys()).intersection(colorsp)
+            if len(colors):
+                if not isinstance(bdryfct[next(iter(colors))],dict):
                     msg = """
                     For Pressure b.c. please give a dictionary {p:fct_scal, v:fvt_vec} with fct_scal scalar and fvt_vec a list of dim functions
                     """
-                    raise ValueError(msg+f"\ngiven: {bdryfct[next(iter(colorspres))]=}")
+                    raise ValueError(msg+f"\ngiven: {bdryfct[next(iter(colors))]=}")
                 pfct, vfct = {}, {}
-                for col in colorspres:
+                for col in colors:
                     if 'p' in bdryfct[col].keys() : 
                         if not callable(bdryfct[col]['p']):
                             raise ValueError(f"'vn' must be a function. Given:{bdryfct[col]['p']=}")
@@ -431,14 +405,16 @@ class Stokes(Application):
             # print(f"{id(A)=} {id(B)=}")
         if self.hdivpenalty:
             if not hasattr(self.mesh,'innerfaces'): self.mesh.constructInnerFaces()
-            A += self.femv.computeMatrixRtPenaly(self.hdivpenalty)
+            A += self.femv.computeMatrixHdivPenaly(self.hdivpenalty)
+        if self.divdivparam:
+            A += self.femv.computeMatrixDivDiv(self.divdivparam)
         if not self.pmean:
-            return [A, B]
+            return linalg.SaddlePointSystem(A, B)
         ncells = self.mesh.ncells
         rows = np.zeros(ncells, dtype=int)
         cols = np.arange(0, ncells)
         C = sparse.coo_matrix((self.mesh.dV, (rows, cols)), shape=(1, ncells)).tocsr()
-        return [A,B,C]
+        return linalg.SaddlePointSystem(A, B, C)
     def computeFormMeanPressure(self,dp, dlam, p, lam):
         dlam += self.mesh.dV.dot(p)
         dp += lam*self.mesh.dV
@@ -447,7 +423,7 @@ class Stokes(Application):
         flux = np.zeros(shape=(ncomp,len(colors)))
         vdir = self.femv.interpolateBoundary(colors, bdryfct).ravel()
         for icomp in range(ncomp):
-            flux[icomp] = self.femv.fem.computeBdryNormalFluxNitsche(v[icomp::ncomp], colors, vdir[icomp::ncomp], self.mucell, nitsche_param=self.dirichlet_nitsche)
+            flux[icomp] = self.femv.fem.computeBdryNormalFluxNitsche(v[icomp::ncomp], colors, vdir[icomp::ncomp], self.mucell)
             for i,color in enumerate(colors):
                 faces = self.mesh.bdrylabels[color]
                 cells = self.mesh.cellsOfFaces[faces,0]
@@ -462,7 +438,7 @@ class Stokes(Application):
         cells = self.mesh.cellsOfFaces[faces,0]
         normalsS = self.mesh.normals[faces][:,:ncomp]
         np.add.at(bp, cells, -np.einsum('nk,nk->n', coeff*vdir[faces], normalsS))
-        self.femv.computeRhsNitscheDiffusion(bv, mucell, colors, vdir, ncomp, nitsche_param=self.dirichlet_nitsche)
+        self.femv.computeRhsNitscheDiffusion(bv, mucell, colors, vdir, ncomp)
     def computeRhsBdryNitscheNavierNormal(self, b, colors, mucell, vn):
         bv, bp = b
         ncomp, dim  = self.ncomp, self.mesh.dimension
@@ -473,7 +449,7 @@ class Stokes(Application):
         # normals = normalsS/dS[:,np.newaxis]
         # foc = self.mesh.facesOfCells[cells]
         np.add.at(bp, cells, -dS*vn[faces])
-        self.femv.computeRhsNitscheDiffusionNormal(bv, mucell, colors, vn, ncomp, nitsche_param=self.dirichlet_nitsche)
+        self.femv.computeRhsNitscheDiffusionNormal(bv, mucell, colors, vn, ncomp)
     def computeRhsBdryNitscheNavierTangent(self, b, colors, mucell, gt):
         bv, bp = b
         ncomp, dim  = self.ncomp, self.mesh.dimension
@@ -485,11 +461,11 @@ class Stokes(Application):
     def computeRhsBdryNitschePressureTangent(self, b, colors, mucell, v):
         bv, bp = b
         ncomp, dim  = self.ncomp, self.mesh.dimension
-        self.femv.computeRhsNitscheDiffusion(bv, mucell, colors, v, ncomp, nitsche_param=self.dirichlet_nitsche)
-        self.femv.computeRhsNitscheDiffusionNormal(bv, mucell, colors, -v.ravel(), ncomp, nitsche_param=self.dirichlet_nitsche)
+        self.femv.computeRhsNitscheDiffusion(bv, mucell, colors, v, ncomp)
+        self.femv.computeRhsNitscheDiffusionNormal(bv, mucell, colors, -v.ravel(), ncomp)
     def computeFormBdryNitscheDirichlet(self, dv, dp, v, p, colorsdir, mu):
         ncomp, dim  = self.femv.ncomp, self.mesh.dimension
-        self.femv.computeFormNitscheDiffusion(dv, v, mu, colorsdir, ncomp, nitsche_param=self.dirichlet_nitsche)
+        self.femv.computeFormNitscheDiffusion(dv, v, mu, colorsdir, ncomp)
         faces = self.mesh.bdryFaces(colorsdir)
         cells = self.mesh.cellsOfFaces[faces, 0]
         normalsS = self.mesh.normals[faces][:, :self.ncomp]
@@ -503,7 +479,7 @@ class Stokes(Application):
         raise NotImplementedError()
     def computeMatrixBdryNitscheDirichlet(self, A, B, colors, mucell):
         nfaces, ncells, ncomp, dim  = self.mesh.nfaces, self.mesh.ncells, self.femv.ncomp, self.mesh.dimension
-        A += self.femv.computeMatrixNitscheDiffusion(mucell, colors, ncomp, nitsche_param=self.dirichlet_nitsche)
+        A += self.femv.computeMatrixNitscheDiffusion(mucell, colors, ncomp)
         #grad-div
         faces = self.mesh.bdryFaces(colors)
         cells = self.mesh.cellsOfFaces[faces, 0]
@@ -527,13 +503,13 @@ class Stokes(Application):
         rows = cells.repeat(ncomp).ravel()
         B -= sparse.coo_matrix((normalsS.ravel(), (rows, cols)), shape=(ncells, ncomp*nfaces))
         #vitesses
-        A += self.femv.computeMatrixNitscheDiffusionNormal(mucell, colors, ncomp, nitsche_param=self.dirichlet_nitsche)
+        A += self.femv.computeMatrixNitscheDiffusionNormal(mucell, colors, ncomp)
         A += self.femv.computeMassMatrixBoundary(colors, ncomp, coeff=lambdaR)-self.femv.computeMassMatrixBoundaryNormal(colors, ncomp, coeff=lambdaR)
         return A,B
     def computeMatrixBdryNitschePressure(self, A, B, colors, mucell):
         #vitesses
-        A += self.femv.computeMatrixNitscheDiffusion(mucell, colors, self.ncomp, nitsche_param=self.dirichlet_nitsche)
-        A -= self.femv.computeMatrixNitscheDiffusionNormal(mucell, colors, self.ncomp, nitsche_param=self.dirichlet_nitsche)
+        A += self.femv.computeMatrixNitscheDiffusion(mucell, colors, self.ncomp)
+        A -= self.femv.computeMatrixNitscheDiffusionNormal(mucell, colors, self.ncomp)
         return A,B
     def vectorBoundaryStrong(self, b, bdryfctv, bdrydata, method):
         bv, bp = b
