@@ -10,15 +10,20 @@ import numpy as np
 import scipy.linalg as linalg
 import scipy.sparse as sparse
 from simfempy.tools import barycentric, npext, checkmmatrix
-from simfempy import fems
 from simfempy.meshes import move, plotmesh
+from simfempy.fems import p1general
+import simfempy.fems.data, simfempy.fems.rt0
 
 #=================================================================#
-class CR1(fems.fem.Fem):
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-        self.dirichlet_strong = 1
-        self.dirichlet_nitsche = 2
+class CR1(p1general.P1general):
+    def __init__(self, kwargs={}, mesh=None):
+        super().__init__(mesh=mesh)
+        for p,v in zip(['masslumpedvol', 'masslumpedbdry'], [False, True]):
+            self.params_bool[p] = kwargs.pop(p, v)
+        for p, v in zip(['dirichletmethod', 'convmethod'], ['strong', 'upw']):
+            self.params_str[p] = kwargs.pop(p, v)
+        if self.params_str['dirichletmethod'] == 'nitsche':
+            self.params_float['nitscheparam'] = kwargs.pop('nitscheparam', 2)
     def setMesh(self, mesh):
         super().setMesh(mesh)
         self.computeStencilCell(self.mesh.facesOfCells)
@@ -38,11 +43,12 @@ class CR1(fems.fem.Fem):
         unodes /= countnodes
         # print(f"{unodes=}")
         return unodes
-    def prepareAdvection(self, beta, scale, method):
-        rt = fems.rt0.RT0(self.mesh)
+    def prepareAdvection(self, beta, scale):
+        method = self.params_str['convmethod']
+        rt = simfempy.fems.rt0.RT0(self.mesh)
         betart = scale*rt.interpolate(beta)
         beta = rt.toCell(betart)
-        convdata = fems.data.ConvectionData(beta=beta, betart=betart)
+        convdata = simfempy.fems.data.ConvectionData(beta=beta, betart=betart)
         dim = self.mesh.dimension
         self.mesh.constructInnerFaces()
         if method == 'upwalg' or method == 'lps':
@@ -58,13 +64,13 @@ class CR1(fems.fem.Fem):
             # print(f"{self.md.mus=}")
             # self.md.plot(self.mesh, beta, type='midpoints')
         elif method == 'upw':
-            md = move.move_midpoints(self.mesh, beta, bound=1/d)
+            md = move.move_midpoints(self.mesh, beta, bound=1/dim)
             # self.md = move.move_midpoint_to_neighbour(self.mesh, betart)
             # self.md = move.move_midpoints(self.mesh, -beta, bound=1/d)
             # self.md = move.move_midpoints(self.mesh, -beta, candidates='all')
             # self.md.plot(self.mesh, beta, type='midpoints')
         elif method == 'upw2':
-            md = move.move_midpoints(self.mesh, -beta, bound=1/d)
+            md = move.move_midpoints(self.mesh, -beta, bound=1/dim)
         else:
             raise ValueError(f"don't know {method=}")
         convdata.md = md
@@ -73,8 +79,8 @@ class CR1(fems.fem.Fem):
         normals, facesOfCells, dV = self.mesh.normals, self.mesh.facesOfCells, self.mesh.dV
         return (normals[facesOfCells].T * self.mesh.sigma.T / dV.T).T
     # strong bc
-    def prepareBoundary(self, colorsdir, colorsflux=[]):
-        bdrydata = fems.data.BdryData()
+    def _prepareBoundary(self, colorsdir, colorsflux=[]):
+        bdrydata = simfempy.fems.data.BdryData()
         bdrydata.facesdirall = np.empty(shape=(0), dtype=np.uint32)
         bdrydata.colorsdir = colorsdir
         for color in colorsdir:
@@ -86,8 +92,11 @@ class CR1(fems.fem.Fem):
             facesdir = self.mesh.bdrylabels[color]
             bdrydata.facesdirflux[color] = facesdir
         return bdrydata
-    def computeRhsNitscheDiffusion(self, b, diffcoff, colorsdir, udir, nitsche_param=None, coeff=1, lumped=False):
-        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
+    def computeRhsNitscheDiffusion(self, b, diffcoff, colorsdir, udir=None, bdrycondfct=None, coeff=1, lumped=False):
+        if self.params_str['dirichletmethod'] != 'nitsche': return
+        if udir is None:
+            udir = self.interpolateBoundary(colorsdir, bdrycondfct)
+        nitsche_param=self.params_float['nitscheparam']
         assert udir.shape[0] == self.mesh.nfaces
         dim, faces = self.mesh.dimension, self.mesh.bdryFaces(colorsdir)
         cells = self.mesh.cellsOfFaces[faces,0]
@@ -96,8 +105,9 @@ class CR1(fems.fem.Fem):
         mat = np.einsum('f,fi,fji->fj', coeff*udir[faces]*diffcoff[cells], normalsS, self.cellgrads[cells, :, :dim])
         np.add.at(b, self.mesh.facesOfCells[cells], -mat)
         self.massDotBoundary(b, f=udir, colors=colorsdir, coeff=coeff*nitsche_param*diffcoff[cells] * dS/dV, lumped=lumped)
-    def computeFormNitscheDiffusion(self, du, u, diffcoff, colorsdir, nitsche_param=None):
-        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
+    def computeFormNitscheDiffusion(self, du, u, diffcoff, colorsdir):
+        if self.params_str['dirichletmethod'] != 'nitsche': return
+        nitsche_param=self.dirichlet_nitsche
         assert u.shape[0] == self.mesh.nfaces
         dim, faces = self.mesh.dimension, self.mesh.bdryFaces(colorsdir)
         cells = self.mesh.cellsOfFaces[faces,0]
@@ -108,9 +118,10 @@ class CR1(fems.fem.Fem):
         mat = np.einsum('f,fk,fjk,fj->f', diffcoff[cells], normalsS, cellgrads, u[foc])
         np.add.at(du, faces, -mat)
         self.massDotBoundary(du, f=u, colors=colorsdir, coeff=nitsche_param*diffcoff[cells]* dS/dV)
-    def computeMatrixNitscheDiffusion(self, diffcoff, colors, nitsche_param=None, coeff=1, lumped=False):
-        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
+    def computeMatrixNitscheDiffusion(self, diffcoff, colors, coeff=1, lumped=False):
         nfaces, ncells, dim, nlocal  = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.nlocal()
+        if self.params_str['dirichletmethod'] != 'nitsche': return sparse.coo_matrix((nfaces,nfaces))
+        nitsche_param=self.params_float['nitscheparam']
         faces = self.mesh.bdryFaces(colors)
         if not isinstance(coeff, (float,int)): assert coeff.shape[0]==faces.shape[0]
         cells = self.mesh.cellsOfFaces[faces, 0]
@@ -125,8 +136,8 @@ class CR1(fems.fem.Fem):
         # AD = sparse.coo_matrix((dS**2/dV,(faces,faces)), shape=(nfaces, nfaces))
         AD = self.computeBdryMassMatrix(colors=colors, coeff=coeff*diffcoff[cells]*nitsche_param*dS/dV, lumped=lumped)
         return AD - AN -AN.T 
-    def computeBdryNormalFluxNitsche(self, u, colors, udir, diffcoff, nitsche_param=None):
-        if nitsche_param==None: nitsche_param=self.dirichlet_nitsche
+    def computeBdryNormalFluxNitsche(self, u, colors, udir, diffcoff):
+        nitsche_param=self.params_float['nitscheparam']
         #TODO correct flux computation Nitsche
         flux= np.zeros(len(colors))
         nfaces, ncells, dim, nlocal  = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.nlocal()
@@ -144,11 +155,15 @@ class CR1(fems.fem.Fem):
             # flux[i] /= np.sum(dS)
         return flux
     def vectorBoundaryStrongEqual(self, du, u, bdrydata):
+        if self.params_str['dirichletmethod']=="nitsche": return
         facesdirall = bdrydata.facesdirall
         du[facesdirall] = u[facesdirall]
     def vectorBoundaryStrongZero(self, du, bdrydata):
+        if self.params_str['dirichletmethod']=="nitsche": return
         du[bdrydata.facesdirall] = 0
-    def vectorBoundaryStrong(self, b, bdrycond, bdrydata, method):
+    def vectorBoundaryStrong(self, b, bdrycond, bdrydata):
+        method = self.params_str['dirichletmethod']
+        if method not in ['strong','new']: return
         facesdirflux, facesinner, facesdirall, colorsdir = bdrydata.facesdirflux, bdrydata.facesinner, bdrydata.facesdirall, bdrydata.colorsdir
         x, y, z = self.mesh.pointsf.T
         for color, faces in facesdirflux.items():
@@ -164,8 +179,9 @@ class CR1(fems.fem.Fem):
             b[facesdirall] = help[facesdirall]
         else:
             b[facesdirall] = bdrydata.A_dir_dir * help[facesdirall]
-    def matrixBoundaryStrong(self, A, bdrydata, method):
-        assert method != 'nitsche'
+    def matrixBoundaryStrong(self, A, bdrydata):
+        method = self.params_str['dirichletmethod']
+        if method not in ['strong','new']: return
         facesdirflux, facesinner, facesdirall, colorsdir = bdrydata.facesdirflux, bdrydata.facesinner, bdrydata.facesdirall, bdrydata.colorsdir
         nfaces = self.mesh.nfaces
         for color, faces in facesdirflux.items():
@@ -210,7 +226,7 @@ class CR1(fems.fem.Fem):
                 mask = foc != faces[:, np.newaxis]
                 fi = foc[mask].reshape(foc.shape[0], foc.shape[1] - 1)
                 normalsS = self.mesh.normals[faces]
-                dS = linalg.norm(normalsS,axis=1)
+                dS = linalg.norm(normalsS, axis=1)
                 normalsS = normalsS/dS[:,np.newaxis]
                 nx, ny, nz = normalsS.T
                 x, y, z = self.mesh.pointsf[faces].T
@@ -249,9 +265,10 @@ class CR1(fems.fem.Fem):
             mass = coeff/(dim+1)*dV.repeat(dim+1)
             rows = self.mesh.facesOfCells.ravel()
             return sparse.coo_matrix((mass, (rows, rows)), shape=(nfaces, nfaces)).tocsr()
-        scalemass = (2-dim) / (dim+1) / (dim+2)
+        scalemass = coeff*(2-dim) / (dim+1) / (dim+2)
         massloc = np.tile(scalemass, (self.nloc,self.nloc))
-        massloc.reshape((self.nloc*self.nloc))[::self.nloc+1] = (2-dim + dim*dim) / (dim+1) / (dim+2)
+        scale = coeff*(2-dim + dim*dim) / (dim+1) / (dim+2)
+        massloc.reshape((self.nloc*self.nloc))[::self.nloc+1] = scale
         mass = np.einsum('n,kl->nkl', dV, massloc).ravel()
         return sparse.coo_matrix((mass, (self.rows, self.cols)), shape=(nfaces, nfaces)).tocsr()
     def computeBdryMassMatrix(self, colors=None, coeff=1, lumped=False):
@@ -304,12 +321,12 @@ class CR1(fems.fem.Fem):
                 mat = np.append(mat, np.einsum('n,kl->nkl', dS, massloc).ravel())
         # print(f"{mat=}")
         return sparse.coo_matrix((mat, (rows, cols)), shape=(nfaces, nfaces)).tocsr()
-    def massDotBoundary(self, b=None, f=None, colors=None, coeff=1, lumped=False):
+    def massDotBoundary(self, b=None, f=None, colors=None, coeff=1, lumped=None):
         #TODO CR1 boundary integrals: can do at ones since last index in facesOfCells is the bdry side:
         # assert np.all(faces == foc[:,-1])
+        if lumped is None: lumped=self.params_bool['masslumpedbdry']
         if colors is None: colors = self.mesh.bdrylabels.keys()
         massloc = barycentric.crbdryothers(self.mesh.dimension)
-        # lumped==False
         if not isinstance(coeff, dict):
             faces = self.mesh.bdryFaces(colors)
             normalsS = self.mesh.normals[faces]
@@ -319,7 +336,10 @@ class CR1(fems.fem.Fem):
             else: dS *= coeff
             if b is None: bsum = np.sum(dS*f[faces])
             else: b[faces] += dS*f[faces]
-            if not lumped:
+            if lumped:
+                if b is None: return bsum
+                else: return b
+            else:
                 ci = self.mesh.cellsOfFaces[faces][:, 0]
                 foc = self.mesh.facesOfCells[ci]
                 mask = foc!=faces[:,np.newaxis]
@@ -328,8 +348,7 @@ class CR1(fems.fem.Fem):
                 if b is None: return bsum+np.sum(r)
                 # print(f"{np.linalg.norm(f[fi])=}")
                 np.add.at(b, fi, r)
-            return b
-
+                return b
         assert(isinstance(coeff, dict))
         for color in colors:
             faces = self.mesh.bdrylabels[color]
@@ -352,9 +371,11 @@ class CR1(fems.fem.Fem):
         A = self.computeMatrixTransportCellWise(data, type='centered')
         # moins bon en L2, mais meilleur en H1
         # A += self.computeMatrixJump(betart)
-        self.diffalg = checkmmatrix.diffudsionForMMatrix(A)
+        self.diffalg = checkmmatrix.diffusionForMMatrix(A)
         return A + self.diffalg
     def computeMatrixTransportUpwind(self, data, method):
+        return self.computeMatrixTransportUpwindAlg(data)
+
         beta, betart, mus = data.beta, data.betart, data.md.mus
         nfaces, ncells, dim, dV = self.mesh.nfaces, self.mesh.ncells, self.mesh.dimension, self.mesh.dV
         normalsS, foc, cof = self.mesh.normals, self.mesh.facesOfCells, self.mesh.cellsOfFaces
@@ -409,9 +430,9 @@ class CR1(fems.fem.Fem):
         A += sparse.coo_matrix((mat, (rows0,cols)), shape=(nfaces, nfaces))
         A += self.computeMatrixJump(betart, mode='primal')
         A -= self.computeBdryMassMatrix(coeff=np.minimum(betart, 0), lumped=False)
-        B = self.computeMatrixTransportSupg(bdrylumped=False, method='supg')
-        if not np.allclose(A.tocsr().A,B.tocsr().A):
-            raise ValueError(f"{A.diagonal()=}\n{B.diagonal()=}")
+        # B = self.computeMatrixTransportSupg(bdrylumped=False, method='supg')
+        # if not np.allclose(A.tocsr().A,B.tocsr().A):
+        #     raise ValueError(f"{A.diagonal()=}\n{B.diagonal()=}")
 
         # A += self.computeMatrixJump(betart, mode='primal')
         #upwind
@@ -602,6 +623,8 @@ class CR1(fems.fem.Fem):
         else: raise ValueError(f"unknown type {type=}")
         np.add.at(du, foc, mat)
         self.massDotBoundary(du, u, coeff=-np.minimum(betart, 0))
+    def computeFormTransportUpwind(self, du, u, data, method):
+        return self.computeFormTransportUpwindAlg(du, u, data)
     def computeFormTransportUpwindAlg(self, du, u, data):
         self.computeFormTransportCellWise(du, u, data, type='centered')
         if hasattr(self,'diffalg'):
@@ -614,6 +637,7 @@ class CR1(fems.fem.Fem):
         self.computeFormJump(du, u, data.betart)
         self.computeFormLps(du, u, data.betart, **kwargs)
     def massDotSupg(self, b, f, data, coeff=1):
+        if self.params_str['convmethod'][:4] != 'supg': return
         dim, facesOfCells, dV = self.mesh.dimension, self.mesh.facesOfCells, self.mesh.dV
         # beta, mus, deltas = beta, self.md.mus, self.md.deltas
         # cellgrads = self.cellgrads[:,:,:dim]
@@ -638,6 +662,24 @@ class CR1(fems.fem.Fem):
         np.add.at(b, facesOfCells, r)
         return b
     # rhs
+    def computeRhsCell(self, b, rhscell):
+        if rhscell is None: return b
+        if isinstance(rhscell,dict):
+            assert set(rhscell.keys())==set(self.mesh.cellsoflabel.keys())
+            dimension, facesOfCells, dV = self.mesh.dimension, self.mesh.facesOfCells, self.mesh.dV
+            scale = 1 / (dimension + 1)
+            return b
+            scale = 1 / (self.mesh.dimension + 1)
+            for label, fct in rhscell.items():
+                if fct is None: continue
+                cells = self.mesh.cellsoflabel[label]
+                xc, yc, zc = self.mesh.pointsc[cells].T
+                bC = scale * fct(xc, yc, zc) * dV[cells]
+                np.add.at(b, facesOfCells, bC)
+        else:
+            fp1 = self.interpolateCell(rhscell)
+            self.massDotCell(b, fp1, coeff=1)
+        return b
     # postprocess
     def computeErrorL2Cell(self, solexact, uh):
         xc, yc, zc = self.mesh.pointsc.T
@@ -690,31 +732,15 @@ class CR1(fems.fem.Fem):
             else:
                 flux[i] = self.comuteFluxOnRobin(u, faces, dS, bdrycond.fct[color], bdrycond.param[color])
         return flux
-    #------------------------------
-    def test(self):
-        import scipy.sparse.linalg as splinalg
-        colors = mesh.bdrylabels.keys()
-        bdrydata = self.prepareBoundary(colorsdir=colors)
-        A = self.computeMatrixDiffusion(coeff=1)
-        A = self.matrixBoundaryStrong(A, bdrydata=bdrydata, method='strong')
-        b = np.zeros(self.nunknowns())
-        rhs = np.vectorize(lambda x,y,z: 1)
-        fp1 = self.interpolateCell(rhs)
-        self.massDotCell(b, fp1, coeff=1)
-        b = self.vectorBoundaryStrongZero(b, bdrydata)
-        return self.tonode(splinalg.spsolve(A, b))
-
 
 # ------------------------------------- #
-
 if __name__ == '__main__':
     from simfempy.meshes import testmeshes
     from simfempy.meshes import plotmesh
     import matplotlib.pyplot as plt
-
     mesh = testmeshes.backwardfacingstep(h=0.2)
     fem = CR1(mesh=mesh)
     u = fem.test()
     plotmesh.meshWithBoundaries(mesh)
-    plotmesh.meshWithData(mesh, point_data={'u':u}, title="P1 Test", alpha=1)
+    plotmesh.meshWithData(mesh, point_data={'u':u}, title="CR1 Test", alpha=1)
     plt.show()
