@@ -12,8 +12,8 @@ from simfempy.fems import femsys, p1
 
 #=================================================================#
 class P1sys(femsys.Femsys):
-    def __init__(self, ncomp, mesh=None):
-        super().__init__(p1.P1(mesh=mesh), ncomp, mesh)
+    def __init__(self, ncomp, kwargs={}, mesh=None):
+        super().__init__(p1.P1(kwargs=kwargs, mesh=mesh), ncomp, mesh)
     def matrixBoundaryStrong(self, A, bdrydata, method):
         nnodes, ncomp = self.mesh.nnodes, self.ncomp
         nodesdir, nodedirall, nodesinner, nodesdirflux = bdrydata.nodesdir, bdrydata.nodedirall, bdrydata.nodesinner, bdrydata.nodesdirflux
@@ -63,6 +63,8 @@ class P1sys(femsys.Femsys):
         for color, nodes in nodesdir.items():
             if color in bdryfct:
                 # dirichlets = bdryfct[color](x[nodes], y[nodes], z[nodes])
+                if len(bdryfct[color])!=ncomp:
+                    raise ValueError(f"{color=} {len(bdryfct[color])=} {ncomp=}")
                 dirichlets = np.vstack([f(x[nodes], y[nodes], z[nodes]) for f in bdryfct[color]])
                 for icomp in range(ncomp):
                     help[icomp + ncomp * nodes] = dirichlets[icomp]
