@@ -15,12 +15,14 @@ class RT0(fems.fem.Fem):
     """
     on suppose que  self.mesh.edgesOfCell[ic, kk] et oppose à elem[ic,kk] !!!
     """
-    def __init__(self, mesh=None, massproj=None):
+    def __init__(self, kwargs={}, mesh=None):
         super().__init__(mesh=mesh)
-        self.massproj=massproj
+        for p, v in zip(['massproj', 'convmethod'], ['standard', 'supg']):
+            self.params_str[p] = kwargs.pop(p, v)
     def setMesh(self, mesh):
         super().setMesh(mesh)
         self.Mtocell = self.toCellMatrix()
+    def nunknowns(self): return self.mesh.nfaces
     def nlocal(self): return self.mesh.dimension+1
     def interpolate(self, f):
         dim = self.mesh.dimension
@@ -35,9 +37,7 @@ class RT0(fems.fem.Fem):
         nfaces, normals = self.mesh.nfaces, self.mesh.normals[:,:dim]
         assert v.shape[0] == dim*nfaces
         nnormals = normals/linalg.norm(normals, axis=1)[:,np.newaxis]
-        fa = np.array([v[i::dim] for i in range(dim)])
-        # print(f"{fa=}")
-        return np.einsum('ni, in -> n', nnormals, fa)
+        return np.einsum('ni, in -> n', nnormals, v.reshape((dim,nfaces), order='F'))
     def toCellMatrix(self):
         ncells, nfaces, normals, sigma, facesofcells = self.mesh.ncells, self.mesh.nfaces, self.mesh.normals, self.mesh.sigma, self.mesh.facesOfCells
         dim, dV, p, pc, simp = self.mesh.dimension, self.mesh.dV, self.mesh.points, self.mesh.pointsc, self.mesh.simplices
@@ -54,9 +54,10 @@ class RT0(fems.fem.Fem):
         return np.einsum('ni,ni,nij,ni -> nj', v[facesofcells], sigma2, pc[:,np.newaxis,:dim]-p[simp,:dim], dS2[facesofcells])
     def constructMass(self, diffinvcell=None):
         ncells, nfaces, normals, sigma, facesofcells = self.mesh.ncells, self.mesh.nfaces, self.mesh.normals, self.mesh.sigma, self.mesh.facesOfCells
-        dim, dV, nloc, simp = self.mesh.dimension, self.mesh.dV, self.nloc, self.mesh.simplices
+        dim, dV, nloc, simp = self.mesh.dimension, self.mesh.dV, self.mesh.dimension+1, self.mesh.simplices
         p, pc, pf = self.mesh.points, self.mesh.pointsc, self.mesh.pointsf
-        if self.massproj is None:
+        massproj = self.params_str['massproj']
+        if massproj == 'standard':
             # RT
             scalea = 1 / dim / dim / (dim + 2) / (dim + 1)
             scaleb = 1 / dim / dim / (dim + 2) * (dim + 1)
@@ -73,13 +74,13 @@ class RT0(fems.fem.Fem):
                 mat = np.einsum("nij, n -> nij", mat, 1/dV)
             else:
                 mat = np.einsum("nij, n -> nij", mat, diffinvcell / dV  )
-            rows = np.repeat(facesofcells, self.nloc).ravel()
-            cols = np.tile(facesofcells, self.nloc).ravel()
+            rows = np.repeat(facesofcells, nloc).ravel()
+            cols = np.tile(facesofcells, nloc).ravel()
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces)).tocsr()
             # print("A (RT)", A)
             return A
 
-        elif self.massproj=="L2":
+        elif massproj=="L2":
             # RT avec projection L2
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)/dim
             ps = p[simp][:,:,:dim]
@@ -92,7 +93,7 @@ class RT0(fems.fem.Fem):
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces)).tocsr()
             # print("A (RTM)", A)
             return A
-        elif self.massproj == "RT_Bar":
+        elif massproj == "RT_Bar":
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             scale = 1/ (dim+1)
             scale = 2/9
@@ -103,7 +104,7 @@ class RT0(fems.fem.Fem):
             cols = np.tile(facesofcells, self.nloc).ravel()
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces))
             return A.tocsr()
-        elif self.massproj == "Bar_RT":
+        elif massproj == "Bar_RT":
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             scale = 1/ (dim+1)
             scale = 2/9
@@ -115,7 +116,7 @@ class RT0(fems.fem.Fem):
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces))
             return A.tocsr().T
 
-        elif self.massproj == "Hat_RT":
+        elif massproj == "Hat_RT":
             # PG de type RT-Hat (Hat aligned with "m")
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             ps = p[simp][:, :, :dim]
@@ -132,7 +133,7 @@ class RT0(fems.fem.Fem):
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces))
             return A.tocsr().T
 
-        elif self.massproj == "Hat_Hat":
+        elif massproj == "Hat_Hat":
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             ps = p[simp][:, :, :dim]
             ps2 = np.transpose(ps, axes=(2, 0, 1))
@@ -148,7 +149,7 @@ class RT0(fems.fem.Fem):
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces))
             return A.tocsr()
 
-        elif self.massproj=="RT_Tilde":
+        elif massproj=="RT_Tilde":
             # PG de type RT-Tilde (Hat aligned with "n")
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             dT = 1/linalg.norm(normals[facesofcells], axis=2)
@@ -170,7 +171,7 @@ class RT0(fems.fem.Fem):
             # print("A (RTxTilde)", A)
             return A
 
-        elif self.massproj=="Tilde_RT":
+        elif massproj=="Tilde_RT":
             # PG de type RT-Tilde (Hat aligned with "n")
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             dT = 1/linalg.norm(normals[facesofcells], axis=2)
@@ -189,7 +190,7 @@ class RT0(fems.fem.Fem):
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces))
             return A.tocsr().T
 
-        elif self.massproj=="HatxRTOLD":
+        elif massproj=="HatxRTOLD":
             # PG de type Tilde-RT
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             ps = p[simp][:, :, :dim]
@@ -207,7 +208,7 @@ class RT0(fems.fem.Fem):
             # print("A (HatxRT)", A)
             return A
 
-        elif self.massproj=="RTxHatOLD":
+        elif massproj=="RTxHatOLD":
             # PG de type RT-Hat (Hat aligned with "m")
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             ps = p[simp][:, :, :dim]
@@ -224,7 +225,7 @@ class RT0(fems.fem.Fem):
             A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(nfaces, nfaces))
             return A.T.tocsr()
 
-        elif self.massproj=="HatxHatOLD":
+        elif massproj=="HatxHatOLD":
             # G de type Tilde-Tilde
             dS = sigma * linalg.norm(normals[facesofcells], axis=2)
             ps = p[simp][:, :, :dim]
@@ -239,65 +240,70 @@ class RT0(fems.fem.Fem):
             return A
 
         else:
-            raise ValueError("unknown type self.massproj={}".format(self.massproj))
+            raise ValueError(f"unknown type {massproj=}")
     def constructDiv(self):
         ncells, nfaces, normals, sigma, facesofcells = self.mesh.ncells, self.mesh.nfaces, self.mesh.normals, self.mesh.sigma, self.mesh.facesOfCells
-        rows = np.repeat(np.arange(ncells), self.nloc)
+        nloc = self.mesh.dimension+1
+        rows = np.repeat(np.arange(ncells), nloc)
         cols = facesofcells.ravel()
         mat =  (sigma*linalg.norm(normals[facesofcells],axis=2)).ravel()
         return  sparse.coo_matrix((mat, (rows, cols)), shape=(ncells, nfaces)).tocsr()
     def reconstruct(self, p, vc, diffinv):
-        nnodes, ncells, dim = self.mesh.nnodes, self.mesh.ncells, self.mesh.dimension
+        nnodes, ncells, dim, simp = self.mesh.nnodes, self.mesh.ncells, self.mesh.dimension, self.mesh.simplices
         if len(diffinv.shape) != 1:
             raise NotImplemented("only scalar diffusion the time being")
-        counts = np.bincount(self.mesh.simplices.reshape(-1))
+        # print(f"{simp=}")
+        counts = np.bincount(simp.reshape(-1).astype(int))
+        # print(f"{counts=}")
         pn2 = np.zeros(nnodes)
-        xdiff = self.mesh.points[self.mesh.simplices, :dim] - self.mesh.pointsc[:, np.newaxis,:dim]
-        rows = np.repeat(self.mesh.simplices,dim)
-        cols = np.repeat(dim*np.arange(ncells),dim*(dim+1)).reshape(ncells * (dim+1), dim) + np.arange(dim)
-        mat = np.einsum("nij, n -> nij", xdiff, diffinv)
-        A = sparse.coo_matrix((mat.reshape(-1), (rows.reshape(-1), cols.reshape(-1))), shape=(nnodes, dim*ncells)).tocsr()
-        np.add.at(pn2, self.mesh.simplices.T, p)
-        pn2 += A*vc
+        xdiff = self.mesh.points[simp, :dim] - self.mesh.pointsc[:, np.newaxis,:dim]
+        # rows = np.repeat(simp,dim)
+        # cols = np.repeat(dim*np.arange(ncells),dim*(dim+1)).reshape(ncells * (dim+1), dim) + np.arange(dim)
+        # mat = np.einsum("nij, n -> nij", xdiff, diffinv)
+        # A = sparse.coo_matrix((mat.reshape(-1), (rows.reshape(-1), cols.reshape(-1))), shape=(nnodes, dim*ncells)).tocsr()
+        np.add.at(pn2, simp, p[:,np.newaxis])
+        assert vc.shape[1]==dim and vc.shape[0]==ncells
+        # raise ValueError(f"{nnodes=} {ncells=} {self.mesh.points.shape=} {xdiff.shape=}")
+        # pn2 += A*vc
+        np.add.at(pn2, simp, np.einsum("nij, n, nj -> ni", xdiff, diffinv, vc))
+        # pn2 += np.einsum("nij, n, nj -> ni", xdiff, diffinv, vc)
         pn2 /= counts
         return pn2
-    def rhsDirichlet(self, faces, ud):
-        return linalg.norm(self.mesh.normals[faces],axis=1) * ud
-    def constructRobin(self, bdrycond, type):
+    # def rhsDirichlet(self, faces, ud):
+    #     return linalg.norm(self.mesh.normals[faces],axis=1) * ud
+    def computeBdryMassMatrix(self, colors, param):
         nfaces = self.mesh.nfaces
         rows = np.empty(shape=(0), dtype=int)
         cols = np.empty(shape=(0), dtype=int)
         mat = np.empty(shape=(0), dtype=float)
-        for color, faces in self.mesh.bdrylabels.items():
-            if bdrycond.type[color] != type: continue
-            if not bdrycond.param[color]: continue
+        for color in colors:
+            faces = self.mesh.bdrylabels[color]
             normalsS = self.mesh.normals[faces]
             dS = linalg.norm(normalsS, axis=1)
             cols = np.append(cols, faces)
             rows = np.append(rows, faces)
-            mat = np.append(mat, 1/bdrycond.param[color] * dS)
+            mat = np.append(mat,  dS/param[color])
         A = sparse.coo_matrix((mat, (rows, cols)), shape=(nfaces, nfaces)).tocsr()
         return A
-    def matrixNeumann(self, A, B, bdrycond):
-        nfaces = self.mesh.nfaces
-        bdrydata = simfempy.fems.bdrydata.BdryData()
+
+    def prepareBoundary(self, colorsneumann):
+        bdrydata = fems.data.BdryData()
         bdrydata.facesneumann = np.empty(shape=(0), dtype=int)
-        bdrydata.colorsneum = bdrycond.colorsOfType("Neumann")
-        for color in bdrydata.colorsneum:
+        for color in colorsneumann:
             bdrydata.facesneumann = np.unique(np.union1d(bdrydata.facesneumann, self.mesh.bdrylabels[color]))
         bdrydata.facesinner = np.setdiff1d(np.arange(self.mesh.nfaces, dtype=int), bdrydata.facesneumann)
-
+        return bdrydata
+    def matrixNeumann(self, A, B, bdrydata):
+        nfaces = self.mesh.nfaces
         bdrydata.B_inner_neum = B[:, :][:, bdrydata.facesneumann]
         help = np.ones(nfaces)
         help[bdrydata.facesneumann] = 0
         help = sparse.dia_matrix((help, 0), shape=(nfaces, nfaces))
         B = B.dot(help)
-
         bdrydata.A_inner_neum = A[bdrydata.facesinner, :][:, bdrydata.facesneumann]
         bdrydata.A_neum_neum = A[bdrydata.facesneumann, :][:, bdrydata.facesneumann]
         help2 = np.zeros((nfaces))
         help2[bdrydata.facesneumann] = 1
         help2 = sparse.dia_matrix((help2, 0), shape=(nfaces, nfaces))
         A = help.dot(A.dot(help)) + help2.dot(A.dot(help2))
-
-        return bdrydata, A, B
+        return A, B, bdrydata
