@@ -1,7 +1,9 @@
 import matplotlib.pyplot as plt
+import numpy as np
 from simfempy.meshes import plotmesh1d, plotmesh2d, plotmesh3d
 import meshio
 
+# TODO ranger plotmesh
 #----------------------------------------------------------------#
 def _getDim(meshdata):
     try:
@@ -197,6 +199,7 @@ def meshWithData(meshdata, **kwargs):
                 plotmesh2d._plotCellsLabels(x, y, simp, xc, yc, ax=ax)
             else: raise NotImplementedError("3d...")
         ax.set_aspect(aspect='equal')
+        if 'title' in kwargs: ax.set_title(kwargs['title'])
         fig.add_subplot(ax)
         count += 1
     for cdn, cd in cell_data.items():
