@@ -4,7 +4,7 @@ Created on Sun Dec  4 18:14:29 2016
 
 @author: becker
 """
-import os, sys, importlib
+import os
 import meshio
 import numpy as np
 from numpy.lib.shape_base import take_along_axis
@@ -46,10 +46,17 @@ class SimplexMesh(object):
         s += f"\ncellsoflabel={list(self.cellsoflabel.keys())}"
         return s
     def __init__(self, mesh, **kwargs):
-        if not isinstance(mesh, meshio.Mesh):
-            raise KeyError(f"Needs a meshio.Mesh, got {type(mesh)}")
+        # if not isinstance(mesh, meshio.Mesh):
+        #     raise KeyError(f"Needs a meshio.Mesh, got {type(mesh)}")
         self.timer = timer.Timer(name="SimplexMesh")
-        self._initMeshPyGmsh(mesh)
+        import importlib
+        from distutils.version import StrictVersion
+        newmeshio = StrictVersion(importlib.metadata.version('meshio')) > StrictVersion('5.0.0')
+        if newmeshio:
+            celltypes = [c.type for c in mesh.cells]
+        else:
+            celltypes = [key for key, cellblock in mesh.cells]
+        self._initMeshPyGmsh(mesh, celltypes)
         self.check()
         # print(self.timer)
     def check(self):
@@ -73,52 +80,45 @@ class SimplexMesh(object):
         faces = self.faces[self.innerfaces]
         fi0_bis = np.empty_like(faces)
         fi1_bis = np.empty_like(faces)
-        # can only serach on by one (and this keeps the order)
+        # can only search on by one (and this keeps the order)
         for i in range(faces.shape[1]):
             fi0_bis[:,i] = self.facesOfCells[ci0][self.simplices[ci0] == faces[:,i][:,np.newaxis]]
             fi1_bis[:,i] = self.facesOfCells[ci1][self.simplices[ci1] == faces[:,i][:,np.newaxis]]
         return fi0_bis, fi1_bis
-        # print(f"{faces=}")
-        # print(f"{self.simplices[ci0]=}")
-        # ind = npext.positionin(faces, self.simplices[ci0])
-        # fi0 =  np.take_along_axis(self.facesOfCells[ci0], ind, axis=1)
-        # ind = npext.positionin(faces, self.simplices[ci1])
-        # fi1 =  np.take_along_axis(self.facesOfCells[ci1], ind, axis=1)
-        # return fi0, fi1
-        # if not np.all(fi0==fi0_bis):
-        #     diff = np.any(fi0 != fi0_bis, axis=1)
-        #     print(f"{fi0[diff]=}\n{fi0_bis[diff]=}")
-        #     print(f"\n{self.simplices[ci0][diff]=}\n{faces[diff]=}")
-        #     assert 0
-        # if not np.all(fi1==fi1_bis):
-        #     diff = np.any(fi1 != fi1_bis, axis=1)
-        #     print(f"{fi1[diff]=}\n{fi1_bis[diff]=}")
-        #     print(f"\n{self.simplices[ci0][diff]=}\n{faces[diff]=}")
-        #     assert 0
-    def _initMeshPyGmsh(self, mesh):
+    def _initMeshPyGmsh(self, mesh, celltypes):
         self.pygmsh = mesh
-        self.celltypes = [key for key, cellblock in mesh.cells]
-        assert self.celltypes==list(mesh.cells_dict.keys())
+        assert celltypes==list(mesh.cells_dict.keys())
         # for key, cellblock in cells: keys.append(key)
-        # print("self.celltypes", self.celltypes)
-        if 'tetra' in self.celltypes:
+        # print("celltypes", celltypes)
+        if 'tetra' in celltypes:
             self.dimension = 3
             self.simplicesname, self.facesname = 'tetra', 'triangle'
-        elif 'triangle' in self.celltypes:
+        elif 'triangle' in celltypes:
             self.dimension = 2
             self.simplicesname, self.facesname = 'triangle', 'line'
-        elif 'line' in self.celltypes:
+        elif 'line' in celltypes:
             self.dimension = 1
             self.simplicesname, self.facesname = 'line', 'vertex'
         else:
-            raise ValueError(f"something wrong {self.celltypes=} {mesh=}")
-        for key, cellblock in mesh.cells:
-            # print(f"{key=} {cellblock=}")
-            if key == self.simplicesname: self.simplices = cellblock
-            elif key == self.facesname:
-                self.facesdata = cellblock
-            else:
-                continue
+            raise ValueError(f"something wrong {celltypes=} {mesh=}")
+        if isinstance(mesh.cells, dict):
+            for key, cellblock in mesh.cells:
+                # print(f"{key=} {cellblock=}")
+                if key == self.simplicesname:
+                    self.simplices = cellblock
+                elif key == self.facesname:
+                    self.facesdata = cellblock
+                else:
+                    continue
+        else:
+            for cells in mesh.cells:
+                # print(f"{key=} {cellblock=}")
+                if cells.type == self.simplicesname:
+                    self.simplices = cells.data
+                elif cells.type == self.facesname:
+                    self.facesdata = cells.data
+                else:
+                    continue
         if not hasattr(self,"simplices") or not hasattr(self,"facesdata"):
             raise ValueError(f"something wrong {self=}")
         assert np.all(self.facesdata==mesh.cells_dict[self.facesname])
@@ -128,8 +128,8 @@ class SimplexMesh(object):
         # eliminate drangling points
         nnp = len(np.unique(self.simplices))
         if not np.all(np.unique(self.simplices)==np.arange(nnp)):
-            msg = f"points in simplices {nnp} but {self.points.shape=}"
-            msg += f"\n{self.celltypes=}\n{mesh.cell_sets=}"
+            msg = f"*** points in simplices {nnp} but {mesh.points.shape=}"
+            msg += f"\n{celltypes=}\n{mesh.cell_sets=}"
             msg += f"\n{np.unique(self.simplices)=}"
             msg += f"\n{mesh.cells_dict=}"
             raise ValueError(msg)
@@ -145,7 +145,7 @@ class SimplexMesh(object):
         self.timer.add("_constructNormalsAndAreas")
         # self.cell_sets = mesh.cell_sets
         # print(f"{mesh.cell_sets_dict=}")
-        bdrylabelsgmsh = self._initMeshPyGmsh7(mesh.cell_sets, mesh.cells_dict)
+        bdrylabelsgmsh = self._initMeshPyGmsh7(mesh.cell_sets, mesh.cells_dict, celltypes)
         self.timer.add("_initMeshPyGmsh7")
         # boundaries
         # self._constructBoundaryFaces7(bdryfacesgmshlist, bdrylabelsgmsh)
@@ -153,30 +153,32 @@ class SimplexMesh(object):
         self.timer.add("_constructBoundaryFaces7")
         # print(f"{self.bdrylabels.keys()=}")
         #TODO : remplacer -1 par nan dans les indices
-    def _initMeshPyGmsh7(self, cell_sets, cells_dict):
+    def _initMeshPyGmsh7(self, cell_sets, cells_dict, celltypes):
+        # print(f"{cell_sets=}")
+        # print(f"{cells_dict=}")
         # cell_sets: dict label --> list of None or np.array for each cell_type
         # the indices of the np.array are not the cellids !
         # ???
         # print(f"{cell_sets=}")
         typesoflabel = {}
-        sizes = {key:0 for key in self.celltypes}
-        cellsoflabel = {key:{} for key in self.celltypes}
+        sizes = {key:0 for key in celltypes}
+        cellsoflabel = {key:{} for key in celltypes}
         ctorderd = []
         for label, cb in cell_sets.items():
             if label=='gmsh:bounding_entities': continue
             # print(f"{label=} {cb=}")
-            if len(cb) != len(self.celltypes): raise KeyError(f"mismatch {label=}")
-            for celltype, info in zip(self.celltypes, cb):
+            if len(cb) != len(celltypes): raise KeyError(f"mismatch {label=}")
+            for celltype, info in zip(celltypes, cb):
                 # only one is supposed to be not None
                 if info is not None:
                     try: ilabel=int(label)
                     except: raise ValueError(f"cannot convert to int {label=} {cell_sets=}")
                     cellsoflabel[celltype][ilabel] = info
-                    # print(f"{label=} {type(label)=} {info=}")
+                    # print(f"{label=} {celltype=} {info=}")
                     sizes[celltype] += info.shape[0]
                     typesoflabel[ilabel] = celltype
                     ctorderd.append(celltype)
-        # print(f"{self.celltypes=}\n{cellsoflabel=}")
+        # print(f"{celltypes=}\n{cellsoflabel=}")
         #correcting the numbering in cell_sets
         n = 0
         for ct in list(dict.fromkeys(ctorderd)):
@@ -188,6 +190,7 @@ class SimplexMesh(object):
         self.linesoflabel = {k:cells_dict['line'][v] for k,v in cellsoflabel['line'].items()}
         if self.facesname not in cellsoflabel:
             raise ValueError(f"{self.facesname=} not in {cellsoflabel=}")
+        # print(f"{self.cellsoflabel=}\n{cellsoflabel[self.facesname]=}")
         return cellsoflabel[self.facesname]
     def _constructFacesFromSimplices(self):
         simplices = self.simplices
@@ -225,30 +228,48 @@ class SimplexMesh(object):
         i = -1*np.ones(self.nfaces, dtype=self.facesOfCells.dtype)
         i[unique[1:]] = i2
         self.cellsOfFaces =np.vstack([i0,i]).T
-
-    def _constructBoundaryFaces7(self, bdryfacesgmsh, bdrylabelsgmsh):
+    def _constructBoundaryFaces7(self, facesgmsh, physlabelsgmsh):
         # t = timer.Timer("_constructBoundaryFaces7")
         # bdries
-        # bdryfacesgmsh may contains interior edges for len(celllabels)>1
+        # facesgmsh may contains interior edges for len(celllabels)>1
         # sort along last axis
-        bdryfacesgmsh = np.sort(bdryfacesgmsh)
+        facesgmsh = np.sort(facesgmsh)
         bdryids = np.flatnonzero(self.cellsOfFaces[:,1] == -1)
         bdryfaces = np.sort(self.faces[bdryids],axis=1)
+        # print(f"{facesgmsh=}")
+        # print(f"{physlabelsgmsh=}")
+        # print(f"{bdryfaces=}")
         nnpc = self.simplices.shape[1]
         s = "{0}" + (nnpc-2)*", {0}"
-        dtb = s.format(bdryfacesgmsh.dtype)
+        dtb = s.format(facesgmsh.dtype)
         dtf = s.format(bdryfaces.dtype)
         order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
         # t.add("a")
         if self.dimension==1:
-            bp = np.argsort(bdryfacesgmsh.view(dtb), axis=0).ravel()
+            bp = np.argsort(facesgmsh.view(dtb), axis=0).ravel()
             fp = np.argsort(bdryfaces.view(dtf), axis=0).ravel()
         else:
-            bp = np.argsort(bdryfacesgmsh.view(dtb), order=order, axis=0).ravel()
+            bp = np.argsort(facesgmsh.view(dtb), order=order, axis=0).ravel()
             fp = np.argsort(bdryfaces.view(dtf), order=order, axis=0).ravel()
-        assert np.all(bdryfaces[fp]==bdryfacesgmsh[bp])
+        # if not np.all(bdryfaces[fp]==facesgmsh[bp][:len(fp)]):
+        #     raise ValueError(f"{bdryfaces[fp]=}\n{facesgmsh[bp][:len(fp)]=}")
         bpi = np.argsort(bp)
-        self.bdrylabels = {col:bdryids[fp[bpi[cb]]] for col, cb in bdrylabelsgmsh.items()}
+        # self.bdrylabels = {col:bdryids[fp[bpi[cb]]] for col, cb in physlabelsgmsh.items()}
+        indices = (facesgmsh[bp, None] == bdryfaces[fp]).all(axis=-1).any(axis=-1)
+        binv = np.argsort(bp[indices])
+        bp2 = bp[indices]
+        binv = np.empty_like(bp)
+        binv[bp2] = np.arange(len(bp2))
+        # if not np.all(binv == np.argsort(bp[indices])):
+        #     print(f"{indices=}\n{binv=} {np.argsort(bp[indices])=}")
+        self.bdrylabels = {}        
+        for col, cb in physlabelsgmsh.items():
+            # if indices[bpi[cb[0]]]:
+            if np.all(indices[bpi[cb]]):
+                self.bdrylabels[int(col)] = bdryids[fp[binv[cb]]]
+            else:
+                assert not indices[bpi[cb[0]]]
+            # self.bdrylabels[col] = bdryids[fp[bpi[cb]]]
         # print(f"{bp=}")
         # print(f"{fp=}")
 #https://stackoverflow.com/questions/51352527/check-for-identical-rows-in-different-numpy-arrays
@@ -353,18 +374,8 @@ class SimplexMesh(object):
     # ----------------------------------------------------------------#
     def write(self, filename, dirname = None, data=None):
         filename = self._getfilename(filename, dirname)
-        if self.dimension == 1:
-            cells = {'lines': self.simplices}
-            cells['vertex'] = self.facesdata
-        elif self.dimension ==2:
-            # cells = {'triangle': self.simplices}
-            # cells['line'] = self.facesdata
-            cells = [('triangle', self.simplices), ('line', self.facesdata)]
-            # cells = [('triangle', self.simplices)]
-        else:
-            cells = [('tetra', self.simplices)]
-            # cells = [('tetra', self.simplices), ('triangle', self.facesdata)]
-        # meshio.write_points_cells(filename, self.points, cells, file_format="gmsh")
+        cells = {self.simplicesname: self.simplices}
+        # cells = {self.simplicesname: self.simplices, self.facesname: self.facesdata}
         args = {'points': self.points, 'cells':cells}
         if data is not None:
             if 'point' in data:
