@@ -19,7 +19,7 @@ def test(dim, **kwargs):
     if dim==2:
         data.ncomp=2
         createMesh = testmeshes.unitsquare
-        colors = [1000,1001,1003]
+        colors = [1000,1001,1002,1003]
         colorsneu = [1000]
         #TODO cl navier faux pour deux bords ?!
         colorsnav = [1001]
@@ -31,10 +31,14 @@ def test(dim, **kwargs):
         colorsneu = [103]
         colorsnav = [105]
         colorsp = [101]
+        # colorsneu = colorsp = []
+    colorsnav = []
+    colorsp = []
+    # TODO Navier donne pas solution pour Linear (mais p)
     colorsdir = [col for col in colors if col not in colorsnav and col not in colorsp and col not in colorsneu]
     if 'strong' in paramsdict['dirichletmethod']:
-        colorsdir.append(*colorsnav)
-        colorsdir.append(*colorsp)
+        if len(colorsnav): colorsdir.append(*colorsnav)
+        if len(colorsp): colorsdir.append(*colorsp)
         colorsnav=[]
         colorsp=[]
     data.bdrycond.set("Dirichlet", colorsdir)
@@ -43,8 +47,11 @@ def test(dim, **kwargs):
     data.bdrycond.set("Pressure", colorsp)
     data.postproc.set(name='bdrypmean', type='bdry_pmean', colors=colorsneu)
     data.postproc.set(name='bdrynflux', type='bdry_nflux', colors=colorsdir)
-    linearsolver = kwargs.pop('linearsolver', 'iter')
-    applicationargs= {'problemdata': data, 'exactsolution': exactsolution, 'linearsolver': linearsolver}
+    applicationargs= {'problemdata': data, 'exactsolution': exactsolution}
+    applicationargs['scalels'] = True
+    solvers=['spsolve', 'scipy_gcrotmk@full@200@1', 'scipy_lgmres@full@200@1', 'schur|diag@pyamg_cg@@9@0']
+    applicationargs['linearsolver'] = kwargs.pop('linearsolver', solvers[0])
+
     # applicationargs['mode'] = 'newton'
     comp =  CompareMethods(createMesh=createMesh, paramsdict=paramsdict, application=Stokes, applicationargs=applicationargs, **kwargs)
     return comp.compare()
@@ -55,8 +62,9 @@ def test(dim, **kwargs):
 if __name__ == '__main__':
     # test(dim=2, exactsolution=[["x**2-y","-2*x*y+x**2"],"x*y"], dirichletmethod='nitsche', niter=6, plotsolution=False, linearsolver='iter_gcrotmk')
     # test(dim=3, exactsolution=[["x**2-y+2","-2*x*y+x**2","x+y"],"x*y*z"], dirichletmethod='nitsche', niter=5, plotsolution=False, linearsolver='iter_gcrotmk')
-    # test(dim=2, exactsolution="Linear", niter=3, dirichletmethod='nitsche', plotsolution=True, linearsolver='umf')
-    # test(dim=2, exactsolution="Quadratic", niter=7, dirichletmethod='nitsche', plotsolution=True, linearsolver='umf')
-    # test(dim=2, exactsolution=[["1.0","0.0"],"10"], niter=3, dirichletmethod='nitsche', plotsolution=True, linearsolver='umf')
-    test(dim=3, exactsolution=[["-z","x","x+y"],"11"], niter=3, dirichletmethod=['nitsche'], plotsolution=False, linearsolver='umf')
-    # test(dim=2, exactsolution=[["-y","x"],"10"], niter=3, dirichletmethod='nitsche', plotsolution=False, linearsolver='umf')
+    test(dim=2, exactsolution="Linear", niter=3, dirichletmethod=['nitsche'], plotsolution=True, linearsolver='spsolve')
+    # test(dim=2, exactsolution="Quadratic", niter=7, dirichletmethod='nitsche', plotsolution=True, linearsolver='spsolve')
+    # test(dim=2, exactsolution=[["1.0","0.0"],"10"], niter=3, dirichletmethod='nitsche', plotsolution=True, linearsolver='spsolve')
+    # test(dim=3, exactsolution=[["-z","x","x+y"],"11"], niter=3, dirichletmethod=['nitsche'], linearsolver='spsolve', plotsolution=False)
+    # test(dim=3, exactsolution=[["-z","x","x+y"],"11"], niter=3, dirichletmethod=['nitsche'], plotsolution=False)
+    # test(dim=2, exactsolution=[["-y","x"],"10"], niter=3, dirichletmethod='nitsche', plotsolution=False, linearsolver='spsolve')
