@@ -35,6 +35,7 @@ class AnalyticalFunction():
         self.fct_x = []
         self.fct_xx = []
         for i in range(dim):
+            self.fct_xxx = np.vectorize(sympy.lambdify(symbc, sympy.diff(expr, s[0], 3)),otypes=[float])
             self.fct_xxxx = np.vectorize(sympy.lambdify(symbc, sympy.diff(expr, s[0], 4)),otypes=[float])
             if dim==1: fx = sympy.diff(expr, s)
             else: fx = sympy.diff(expr, s[i])
@@ -54,10 +55,12 @@ class AnalyticalFunction():
         return self.fct_x[2](*x)
     def dd(self, i, j, *x):
         return self.fct_xx[i][j](*x)
-    def dddd(self, *x):
+    def xxxx(self, *x):
         return self.fct_xxxx(*x)
     def xx(self, *x):
         return self.fct_xx[0][0](*x)
+    def xxx(self, *x):
+        return self.fct_xxx(*x)
     def xy(self, *x):
         return self.fct_xx[0][1](*x)
     def xz(self, *x):
