@@ -11,6 +11,13 @@ def tensor(d, k):
         A[i] = np.prod(scipy.special.factorial(np.bincount(i)))/facd
     return A
 
+# ------------------------------------- #
+def crbdryothers(d):
+    massloc = -np.ones(shape=(d, d)) / (d + 1)
+    massloc[np.diag_indices(d)] = (d - 1) / (d + 1)
+    return massloc
+
+
 # ------------------------------------- #
 if __name__ == '__main__':
     print(f"{tensor(d=3, k=2)=}")
