@@ -3,9 +3,9 @@
 Created on Sun Dec  4 18:14:29 2016
 @author: becker
 """
+import os, shutil
 import numpy as np
 import matplotlib.pyplot as plt
-import pygmsh
 from simfempy.tools.latexwriter import LatexWriter
 import simfempy.meshes.pygmshext
 #=================================================================#
@@ -31,11 +31,13 @@ class CompareMethods(object):
       verb: in [0,5]
     """
     def __init__(self, **kwargs):
-        self.dirname = "Results"
-        if 'clean' in kwargs and kwargs.pop("clean")==True:
-            import os, shutil
-            try: shutil.rmtree(os.getcwd() + os.sep + self.dirname)
+        self.fullmethodsname = kwargs.pop("fullmethodsname", True)
+        self.dirname = os.getcwd() + os.sep +"Results"
+        if kwargs.pop("clean",True):
+            try: shutil.rmtree(self.dirname)
             except: pass
+        if not os.path.isdir(self.dirname): os.mkdir(self.dirname)
+
         self.verbose = kwargs.pop("verbose", 1)
         self.latex = kwargs.pop("latex", True)
         self.plotsolution = kwargs.pop("plotsolution", False)
@@ -72,31 +74,42 @@ class CompareMethods(object):
             # self.params = kwargs.pop("params", None)
             self.params = self.paramsdict[self.paramname]
             self.gmshrefine = False
-        self.methods = kwargs.pop("methods", None)
-        if self.methods == None:
+        if 'methods' in kwargs:
+            self.methods = kwargs.pop("methods")
+        else:
             requiredargs = ['application', 'applicationargs']
             for requiredarg in requiredargs:
                 if not requiredarg in kwargs:
                     raise ValueError("need 'application' (class) and 'applicationargs' (dict) and  'paramsdict' (dict)")
             self._definemethods(kwargs.pop("application"), kwargs.pop("applicationargs"))
+        if len(kwargs.keys()):
+            raise ValueError(f"*** unused arguments {kwargs=}")
     def _definemethods(self, application, applicationargs):
+        if not 'problemdata' in applicationargs:
+            raise KeyError(f"'problemdata' should be set in 'applicationargs'")
         import itertools
         paramsdict = self.paramsdict
         if self.paramname in paramsdict: paramsdict.pop(self.paramname)
         for pname,params in paramsdict.items():
             if isinstance(params, str): paramsdict[pname] = [params]
-        paramsprod = list(itertools.product(*paramsdict.values()))
-        paramslist = [{k:params[i] for i,k in enumerate(paramsdict)} for params in paramsprod]
-        if not 'problemdata' in applicationargs:
-            raise KeyError(f"'problemdata' should be set in 'applicationargs'")
+        # paramsprod = list(itertools.product(*paramsdict.values()))
+        # paramslist = [{k:params[i] for i,k in enumerate(paramsdict)} for params in paramsprod]
+        from simfempy.tools import tools
+        paramslist = tools.dictproduct(paramsdict)
+        #TODO virer itertools ici
         self.methods = {}
+        import copy
         for p in paramslist:
             name = ''
+            applicationargs2 = copy.deepcopy(applicationargs)
             for pname, param in p.items():
-                if len(paramsdict[pname])>1: name += str(param)
-                applicationargs[pname] = param
-            self.methods[name] = application(**applicationargs)
-           
+                ps = pname.split('@')
+                if self.fullmethodsname or len(paramsdict[pname])>1: name += str(param)
+                if len(ps)>1:
+                    exec(f"applicationargs2['problemdata'].params.{ps[1]}['{ps[0]}']={param}")
+                else:
+                    applicationargs2[pname] = param
+            self.methods[name] = application(**applicationargs2)
     def _mesh_from_geom_or_fct(self, h=None):
         if h is None:
             if self.createMesh is not None: return self.createMesh()
@@ -113,7 +126,6 @@ class CompareMethods(object):
     def compare(self, **kwargs):
         if (self.gmshrefine or self.paramname != "ncells") and self.mesh is None:
             mesh = self._mesh_from_geom_or_fct()
-
         if self.plotsolution:
             import matplotlib.gridspec as gridspec
             fig = plt.figure(figsize=(10, 8))
@@ -138,6 +150,7 @@ class CompareMethods(object):
                     method.paramname = param
                     # method.setParameter(self.paramname, param)
                 result = method.solve(self.dirname)
+                # print(f"{result=}")
                 if self.plotsolution:
                     from simfempy.meshes import plotmesh
                     suptitle = "{}={}".format(self.paramname, parameters[-1])
@@ -186,12 +199,13 @@ class CompareMethods(object):
             else:
                 self.infos[key2][name][iter] = np.sum(info2)
 
-    def generateLatex(self, names, paramname, parameters, infos):
-        mesh = self.createMesh.__name__
-        title = f"mesh({mesh})\\\\"
-        for name, method in self.methods.items():
-            title += f"{name}\\\\"
-        title = title[:-2]
+    def generateLatex(self, names, paramname, parameters, infos, title=None):
+        if title is None:
+            title = self.createMesh.__name__
+            # title = f"mesh({mesh})\\\\"
+            # for name, method in self.methods.items():
+            #     title += f"{name}\\\\"
+            # title = title[:-2]
         # print("title = ", title)
         latexwriter = LatexWriter(dirname=self.dirname, title=title, author=self.__class__.__name__)
         for key, val in infos.items():
@@ -199,30 +213,22 @@ class CompareMethods(object):
             keysplit = key.split('_')
             if key == 'iter':
                 newdict={}
-                valformat={}
                 for key2, val2 in val.items():
                     for name in names:
                         keyname = "{}-{}".format(key2, name)
-                        # print(f"{keyname=} {val2[name]=}")
                         newdict[keyname] = val2[name]
-                        if np.issubdtype(val2[name].dtype, np.integer): valformat[keyname]='3d'
-                        elif np.issubdtype(val2[name].dtype, np.float): valformat[keyname]='5.1f'
-                        else:  raise ValueError(f"{val2[name].dtype=}")
-                # print(f"{valformat=}")                
                 kwargs['name'] = '{}'.format(key)
                 kwargs['values'] = newdict
-                kwargs['valformat'] = valformat
-                # print(f"{kwargs=}")
                 latexwriter.append(**kwargs)
             elif key == 'timer':
+                sumdict = {name:np.zeros(len(parameters)) for name in names}
                 for name in names:
                     newdict={}
                     for key2, val2 in val.items():
-                        newdict["{}".format(key2)] = val2[name]
-                    kwargs['name'] = '{}-{}'.format(key, name)
-                    kwargs['values'] = newdict
-                    kwargs['percentage'] = True
-                    latexwriter.append(**kwargs)
+                        sumdict[name] += val2[name]
+                        newdict[key2] = val2[name]
+                    latexwriter.append(**kwargs, name = f"{key}-{name}", values=newdict, percentage=True)
+                latexwriter.append(**kwargs, name=key, values=sumdict)
             else:
                 iserr = len(keysplit) >= 2 and keysplit[0] == 'err'
                 kwargs['redrate'] = iserr and (paramname=="ncells")
