@@ -22,18 +22,20 @@ class TableData(object):
         return s
     def _getformat(self,v):
         assert len(v)
-        if isinstance(v[0], (int,np.integer)):
+        isint = (isinstance(v,list) and isinstance(v[0], (int, np.integer))) or (isinstance(v,np.ndarray) and np.issubdtype(v.dtype, np.integer))
+        isfloat = (isinstance(v,list) and isinstance(v[0], (float, float))) or (isinstance(v,np.ndarray) and np.issubdtype(v[0], float))
+        if isint:
             format = "{:15d}"
-        elif isinstance(v[0], str):
-            format = "{:15s}"
-        elif isinstance(v[0], float):
+        elif isfloat:
             v = np.asarray(v)
             if np.all(np.abs(v) < 100) and np.abs(v).min()>0.01:
                 format = "{:10.2f}"
             else:
                 format = "{:10.2e}"
+        elif isinstance(v[0], str):
+            format = "{:15s}"
         else:
-            raise ValueError(f"cannot find instance of {v=} {type(v[0])=}")
+            raise ValueError(f"cannot find instance of {v=} {type(v[0])=} {isinstance(v,np.ndarray)=} {np.issubdtype(v.dtype,np.float)=}")
         return format
     def __init__(self, **kwargs):
         values = kwargs.pop('values')
@@ -85,8 +87,7 @@ class TableData(object):
             for i in range(1,len(n)):
                 valorder[i] = abs(values[key][i]-values[key][i-1])
             values[key2] = valorder
-            self.valformat[key2] = self.valformat[key]
-            self.valformat[key2] = self.valformat[key]
+            self.valformat[key2] = "{:8.2f}"
     def computeReductionRate(self, dim, diff=False):
         n, values, keys = self.n, self.values, list(self.values.keys())
         if not isinstance(n[0],(int,float)): raise ValueError("n must be int or float")
@@ -114,17 +115,15 @@ class TableData(object):
 class LatexWriter(object):
     # def __init__(self, dirname="Resultslatextest", filename=None):
     def __init__(self, **kwargs):
-        self.dirname = "Resultslatextest"
-        if "dirname" in kwargs: self.dirname = kwargs.pop("dirname")
-        filename = self.dirname + ".tex"
-        if "filename" in kwargs: filename = kwargs.pop("filename")
+        self.author = kwargs.pop("author".replace('_',r'\_'), self.__class__.__name__)
+        self.title = kwargs.pop("title".replace('_',r'\_'), "No title given")
+        self.dirname = kwargs.pop("dirname", "Resultslatextest")
+        filename = kwargs.pop("filename", self.title.replace(' ',r'\_'))
         if filename[-4:] != '.tex': filename += '.tex'
         self.dirname += os.sep + "tex"
         if not os.path.isdir(self.dirname): os.makedirs(self.dirname)
         self.latexfilename = os.path.join(self.dirname, filename)
-        self.author, self.title = self.__class__.__name__, "No title given"
-        if "title" in kwargs: self.title = kwargs.pop("title")
-        if "author" in kwargs: self.author = kwargs.pop("author")
+        # raise ValueError(f"{self.latexfilename=}")
         self.sep = '%' + 30*'='+'\n'
         self.data = {}
         self.countdata = 0
@@ -162,7 +161,7 @@ class LatexWriter(object):
         self.latexfile.close()
     def writeTable(self, name, tabledata, sort=False):
         n, nname, nformat, values, valformat = tabledata.n, tabledata.nname, tabledata.nformat, tabledata.values, tabledata.valformat
-        nname = nname.replace('_', '')
+        nname = nname.replace('_', r'\_')
         name = name.replace('_', '')
         keys_to_write = sorted(values.keys()) if sort else list(values.keys())
         size = len(keys_to_write)
@@ -171,21 +170,32 @@ class LatexWriter(object):
         texta += 'r|' + size*'|r' + '}\n'
         self.latexfile.write(texta)
         if self.rotatenames:
-            itemformated = "\sw{%s} &" %nname
+            # itemformated = "\sw{%s} &" %nname
+            itemformated = f"\sw{{{nname}}} &"
             for i in range(size-1):
-                itemformated += "\sw{%s} &" %keys_to_write[i].replace('_','')
-            itemformated += "\sw{%s}\\\\\\hline\hline\n" %keys_to_write[size-1].replace('_','')
+                # itemformated += "\sw{%s} &" %keys_to_write[i].replace('_','\_')
+                k = keys_to_write[i].replace('_', r'\_')
+                itemformated += f"\sw{{{k}}} &"
+            # itemformated += "\sw{%s}\\\\\\hline\hline\n" %keys_to_write[size-1].replace('_','\_')
+            k = keys_to_write[size-1].replace('_', '\_')
+            itemformated += f"\sw{{{k}}}\\\\\\hline\hline\n"
         else:
-            itemformated = "%15s " %nname
+            itemformated = f"{nname:15} "
             for i in range(size):
-                itemformated += " & %15s " %keys_to_write[i].replace('_','')
+                k = keys_to_write[i].replace('_', r'\_')
+                itemformated += f" & {k:15} "
             itemformated += "\\\\\\hline\hline\n"
         self.latexfile.write(itemformated)
         for texline in range(len(n)):
-            itemformated = nformat.format(n[texline])
+            nt = n[texline]
+            if isinstance(nt,str): nt=nt.replace('_',r'\_')
+            itemformated = nformat.format(nt)
             for i in range(size):
                 key = keys_to_write[i]
-                itemformated += '&'+valformat[key].format(values[key][texline])
+                try:
+                    itemformated += '&'+valformat[key].format(values[key][texline])
+                except:
+                    raise ValueError(f"{key=} {values[key]=} {valformat[key]=}")
             itemformated += "\\\\\\hline\n"
             self.latexfile.write(itemformated)
         # texte = '\\end{tabular}\n\\caption{%s}' %(name)
@@ -215,10 +225,9 @@ class LatexWriter(object):
         command = "pdflatex " + filename
         try:
             result = subprocess.call(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-            if result: raise ValueError("command pdflatex not found")
+            if result: raise ValueError("*** command pdflatex not found")
         except:
-            raise ValueError("no pdflatex found") 
-            pass
+            raise ValueError("*** no pdflatex found")
         if platform.system() == "Linux":
             command = "xdg-open "
         elif platform.system() == "Darwin":
@@ -243,7 +252,7 @@ if __name__ == '__main__':
     values2={}
     values2[1] = [1,2,3]
     values2[2] = [4,5,6]
-    latexwriter.append(n=['a','b','c'], nname='letter', values=values2, percentage=True)
+    latexwriter.append(n=['a_a','b','c'], nname='letter', values=values2, percentage=True)
     values3={}
     values3['1'] = np.linspace(1,3,5)
     latexwriter.append(n=np.arange(5), nname= 'toto', values=values3)
